@@ -2314,8 +2314,8 @@ def start_ray_client_server(
     ray_client_server_port: int,
     stdout_file: Optional[int] = None,
     stderr_file: Optional[int] = None,
-    redis_username: Optional[int] = None,
-    redis_password: Optional[int] = None,
+    redis_username: Optional[str] = None,
+    redis_password: Optional[str] = None,
     fate_share: Optional[bool] = None,
     runtime_env_agent_address: Optional[str] = None,
     server_type: str = "proxy",
