@@ -2,6 +2,7 @@ import json
 import logging
 import sys
 from collections import defaultdict
+from threading import Lock
 from typing import Dict, Optional
 
 import ray
@@ -31,32 +32,43 @@ class GlobalState:
         """Create a GlobalState object."""
         # Args used for lazy init of this object.
         self.gcs_options = None
-        self.global_state_accessor = None
+        self._global_state_accessor = None
+        self._init_lock = Lock()
 
-    def _check_connected(self):
-        """Ensure that the object has been initialized before it is used.
-
-        This lazily initializes clients needed for state accessors.
-
-        Raises:
-            RuntimeError: An exception is raised if ray.init() has not been
-                called yet.
+    def _connect_and_get_accessor(self) -> GlobalStateAccessor:
         """
-        if self.gcs_options is not None and self.global_state_accessor is None:
-            self._really_init_global_state()
+        This lazily initializes clients needed for state accessors and returns a connected global state accessor.
 
-        # _really_init_global_state should have set self.global_state_accessor
-        if self.global_state_accessor is None:
-            raise ray.exceptions.RaySystemError(
-                "Ray has not been started yet. You can start Ray with 'ray.init()'."
-            )
+        Returns:
+            GlobalStateAccessor: A connected global state accessor.
+        Raises:
+            RuntimeError: An exception is raised if ray.init() has not been called yet.
+        """
+        with self._init_lock:
+            if self._global_state_accessor is not None:
+                return self._global_state_accessor
+
+            if self.gcs_options is None:
+                raise ray.exceptions.RaySystemError(
+                    "Ray has not been started yet. Trying to use state API before ray.init() has been called."
+                )
+
+            self._global_state_accessor = GlobalStateAccessor(self.gcs_options)
+            connected = self._global_state_accessor.connect()
+            if not connected:
+                self._global_state_accessor = None
+                raise ray.exceptions.RaySystemError(
+                    "Failed to connect to GCS. Please check if the GCS server is running "
+                    "and if this node can connect to the head node."
+                )
+            return self._global_state_accessor
 
     def disconnect(self):
         """Disconnect global state from GCS."""
-        self.gcs_options = None
-        if self.global_state_accessor is not None:
-            self.global_state_accessor.disconnect()
-            self.global_state_accessor = None
+        with self._init_lock:
+            self.gcs_options = None
+            if self._global_state_accessor is not None:
+                self._global_state_accessor = None
 
     def _initialize_global_state(self, gcs_options):
         """Set args for lazily initialization of the GlobalState object.
@@ -71,11 +83,8 @@ class GlobalState:
 
         # Save args for lazy init of global state. This avoids opening extra
         # gcs connections from each worker until needed.
-        self.gcs_options = gcs_options
-
-    def _really_init_global_state(self):
-        self.global_state_accessor = GlobalStateAccessor(self.gcs_options)
-        self.global_state_accessor.connect()
+        with self._init_lock:
+            self.gcs_options = gcs_options
 
     def actor_table(
         self,
@@ -99,11 +108,11 @@ class GlobalState:
         Returns:
             Information from the actor table.
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         if actor_id is not None:
             actor_id = ray.ActorID(hex_to_binary(actor_id))
-            actor_info = self.global_state_accessor.get_actor_info(actor_id)
+            actor_info = accessor.get_actor_info(actor_id)
             if actor_info is None:
                 return {}
             else:
@@ -111,9 +120,7 @@ class GlobalState:
                 return self._gen_actor_info(actor_table_data)
         else:
             validate_actor_state_name(actor_state_name)
-            actor_table = self.global_state_accessor.get_actor_table(
-                job_id, actor_state_name
-            )
+            actor_table = accessor.get_actor_table(job_id, actor_state_name)
             results = {}
             for i in range(len(actor_table)):
                 actor_table_data = gcs_pb2.ActorTableData.FromString(actor_table[i])
@@ -163,9 +170,9 @@ class GlobalState:
         Returns:
             Information about the node in the cluster.
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
-        return self.global_state_accessor.get_node_table()
+        return accessor.get_node_table()
 
     def job_table(self):
         """Fetch and parse the gcs job table.
@@ -179,9 +186,9 @@ class GlobalState:
             - "StartTime" (UNIX timestamp of the start time of this job),
             - "StopTime" (UNIX timestamp of the stop time of this job, if any)
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
-        job_table = self.global_state_accessor.get_job_table(
+        job_table = accessor.get_job_table(
             skip_submission_job_info_field=True, skip_is_running_tasks_field=True
         )
 
@@ -207,9 +214,9 @@ class GlobalState:
         Returns:
             Next job id in the cluster.
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
-        return ray.JobID.from_int(self.global_state_accessor.get_next_job_id())
+        return ray.JobID.from_int(accessor.get_next_job_id())
 
     def profile_events(self):
         """Retrieve and return task profiling events from GCS.
@@ -230,10 +237,10 @@ class GlobalState:
                 ]
             }
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         result = defaultdict(list)
-        task_events = self.global_state_accessor.get_task_events()
+        task_events = accessor.get_task_events()
         for i in range(len(task_events)):
             event = gcs_pb2.TaskEvents.FromString(task_events[i])
             profile = event.profile_events
@@ -264,9 +271,9 @@ class GlobalState:
         return dict(result)
 
     def get_placement_group_by_name(self, placement_group_name, ray_namespace):
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
-        placement_group_info = self.global_state_accessor.get_placement_group_by_name(
+        placement_group_info = accessor.get_placement_group_by_name(
             placement_group_name, ray_namespace
         )
         if placement_group_info is None:
@@ -278,15 +285,13 @@ class GlobalState:
             return self._gen_placement_group_info(placement_group_table_data)
 
     def placement_group_table(self, placement_group_id=None):
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         if placement_group_id is not None:
             placement_group_id = ray.PlacementGroupID(
                 hex_to_binary(placement_group_id.hex())
             )
-            placement_group_info = self.global_state_accessor.get_placement_group_info(
-                placement_group_id
-            )
+            placement_group_info = accessor.get_placement_group_info(placement_group_id)
             if placement_group_info is None:
                 return {}
             else:
@@ -295,9 +300,7 @@ class GlobalState:
                 )
                 return self._gen_placement_group_info(placement_group_info)
         else:
-            placement_group_table = (
-                self.global_state_accessor.get_placement_group_table()
-            )
+            placement_group_table = accessor.get_placement_group_table()
             results = {}
             for placement_group_info in placement_group_table:
                 placement_group_table_data = gcs_pb2.PlacementGroupTableData.FromString(
@@ -456,7 +459,7 @@ class GlobalState:
         # TODO(rkn): This should support viewing just a window of time or a
         # limited number of events.
 
-        self._check_connected()
+        self._connect_and_get_accessor()
 
         # Add a small delay to account for propagation delay of events to the GCS.
         # This should be harmless enough but prevents calls to timeline() from
@@ -537,7 +540,7 @@ class GlobalState:
             If filename is not provided, this returns a list of profiling
                 events. Each profile event is a dictionary.
         """
-        self._check_connected()
+        self._connect_and_get_accessor()
 
         node_id_to_address = {}
         for node_info in self.node_table():
@@ -618,10 +621,10 @@ class GlobalState:
 
     def workers(self):
         """Get a dictionary mapping worker ID to worker information."""
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         # Get all data in worker table
-        worker_table = self.global_state_accessor.get_worker_table()
+        worker_table = accessor.get_worker_table()
         workers_data = {}
         for i in range(len(worker_table)):
             worker_table_data = gcs_pb2.WorkerTableData.FromString(worker_table[i])
@@ -658,15 +661,15 @@ class GlobalState:
         Returns:
              Is operation success
         """
+        accessor = self._connect_and_get_accessor()
+
         worker_data = gcs_pb2.WorkerTableData()
         worker_data.is_alive = True
         worker_data.worker_address.worker_id = worker_id
         worker_data.worker_type = worker_type
         for k, v in worker_info.items():
             worker_data.worker_info[k] = bytes(v, encoding="utf-8")
-        return self.global_state_accessor.add_worker_info(
-            worker_data.SerializeToString()
-        )
+        return accessor.add_worker_info(worker_data.SerializeToString())
 
     def update_worker_debugger_port(self, worker_id, debugger_port):
         """Update the debugger port of a worker.
@@ -678,16 +681,14 @@ class GlobalState:
         Returns:
              Is operation success
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         assert worker_id is not None, "worker_id is not valid"
         assert (
             debugger_port is not None and debugger_port > 0
         ), "debugger_port is not valid"
 
-        return self.global_state_accessor.update_worker_debugger_port(
-            worker_id, debugger_port
-        )
+        return accessor.update_worker_debugger_port(worker_id, debugger_port)
 
     def get_worker_debugger_port(self, worker_id):
         """Get the debugger port of a worker.
@@ -698,11 +699,11 @@ class GlobalState:
         Returns:
              Debugger port of the worker.
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         assert worker_id is not None, "worker_id is not valid"
 
-        return self.global_state_accessor.get_worker_debugger_port(worker_id)
+        return accessor.get_worker_debugger_port(worker_id)
 
     def update_worker_num_paused_threads(self, worker_id, num_paused_threads_delta):
         """Updates the number of paused threads of a worker.
@@ -714,12 +715,12 @@ class GlobalState:
         Returns:
              Is operation success
         """
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
 
         assert worker_id is not None, "worker_id is not valid"
         assert num_paused_threads_delta is not None, "worker_id is not valid"
 
-        return self.global_state_accessor.update_worker_num_paused_threads(
+        return accessor.update_worker_num_paused_threads(
             worker_id, num_paused_threads_delta
         )
 
@@ -733,7 +734,7 @@ class GlobalState:
             A dictionary mapping resource name to the total quantity of that
                 resource in the cluster.
         """
-        self._check_connected()
+        self._connect_and_get_accessor()
 
         # Calculate total resources.
         total_resources = defaultdict(int)
@@ -749,12 +750,10 @@ class GlobalState:
 
     def available_resources_per_node(self):
         """Returns a dictionary mapping node id to available resources."""
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
         available_resources_by_id = {}
 
-        all_available_resources = (
-            self.global_state_accessor.get_all_available_resources()
-        )
+        all_available_resources = accessor.get_all_available_resources()
         for available_resource in all_available_resources:
             message = gcs_pb2.AvailableResources.FromString(available_resource)
             # Calculate available resources for this node.
@@ -769,10 +768,10 @@ class GlobalState:
 
     # returns a dict that maps node_id(hex string) to a dict of {resource_id: capacity}
     def total_resources_per_node(self) -> Dict[str, Dict[str, int]]:
-        self._check_connected()
+        accessor = self._connect_and_get_accessor()
         total_resources_by_node = {}
 
-        all_total_resources = self.global_state_accessor.get_all_total_resources()
+        all_total_resources = accessor.get_all_total_resources()
         for node_total_resources in all_total_resources:
             message = gcs_pb2.TotalResources.FromString(node_total_resources)
             # Calculate total resources for this node.
@@ -799,7 +798,7 @@ class GlobalState:
                 is currently not available (i.e., quantity is 0), it will not
                 be included in this dictionary.
         """
-        self._check_connected()
+        self._connect_and_get_accessor()
 
         available_resources_by_id = self.available_resources_per_node()
 
@@ -813,20 +812,18 @@ class GlobalState:
 
     def get_system_config(self):
         """Get the system config of the cluster."""
-        self._check_connected()
-        return json.loads(self.global_state_accessor.get_system_config())
+        accessor = self._connect_and_get_accessor()
+        return json.loads(accessor.get_system_config())
 
     def get_node_to_connect_for_driver(self, node_ip_address):
         """Get the node to connect for a Ray driver."""
-        self._check_connected()
-        return self.global_state_accessor.get_node_to_connect_for_driver(
-            node_ip_address
-        )
+        accessor = self._connect_and_get_accessor()
+        return accessor.get_node_to_connect_for_driver(node_ip_address)
 
     def get_node(self, node_id: str):
         """Get the node information for a node id."""
-        self._check_connected()
-        return self.global_state_accessor.get_node(node_id)
+        accessor = self._connect_and_get_accessor()
+        return accessor.get_node(node_id)
 
     def get_draining_nodes(self) -> Dict[str, int]:
         """Get all the hex ids of nodes that are being drained
@@ -834,13 +831,13 @@ class GlobalState:
 
         There is no deadline if the timestamp is 0.
         """
-        self._check_connected()
-        return self.global_state_accessor.get_draining_nodes()
+        accessor = self._connect_and_get_accessor()
+        return accessor.get_draining_nodes()
 
     def get_cluster_config(self) -> autoscaler_pb2.ClusterConfig:
         """Get the cluster config of the current cluster."""
-        self._check_connected()
-        serialized_cluster_config = self.global_state_accessor.get_internal_kv(
+        accessor = self._connect_and_get_accessor()
+        serialized_cluster_config = accessor.get_internal_kv(
             ray._raylet.GCS_AUTOSCALER_STATE_NAMESPACE.encode(),
             ray._raylet.GCS_AUTOSCALER_CLUSTER_CONFIG_KEY.encode(),
         )
@@ -900,6 +897,11 @@ class GlobalState:
 
         return result
 
+    def get_actor_info(self, actor_id: ray.ActorID) -> Optional[str]:
+        """Get the actor info for a actor id."""
+        accessor = self._connect_and_get_accessor()
+        return accessor.get_actor_info(actor_id)
+
 
 state = GlobalState()
 """A global object used to access the cluster's global state."""
