@@ -522,16 +522,23 @@ class ReadOnlyProviderConfigReader(IConfigReader):
 
         head_node_type = None
         for node_state in ray_cluster_resource_state.node_states:
-            node_type = format_readonly_node_type(binary_to_hex(node_state.node_id))
+            node_type = node_state.ray_node_type_name
+            if not node_type:
+                node_type = format_readonly_node_type(binary_to_hex(node_state.node_id))
+
             if is_head_node(node_state):
                 head_node_type = node_type
 
-            available_node_types[node_type] = {
-                "resources": dict(node_state.total_resources),
-                "min_workers": 0,
-                "max_workers": 0 if is_head_node(node_state) else 1,
-                "node_config": {},
-            }
+            if node_type not in available_node_types:
+                available_node_types[node_type] = {
+                    "resources": dict(node_state.total_resources),
+                    "min_workers": 0,
+                    "max_workers": 0 if is_head_node(node_state) else 1,
+                    "node_config": {},
+                }
+            elif not is_head_node(node_state):
+                available_node_types[node_type]["max_workers"] += 1
+
         if available_node_types:
             self._configs["available_node_types"].update(available_node_types)
             self._configs["max_workers"] = len(available_node_types)
