@@ -27,6 +27,9 @@ def max_plus_pending(max_resource, pending_resource):
 MAX_PLUS_PENDING_CPUS = max_plus_pending(MAX_CPUS, PENDING_CPUS)
 MAX_PLUS_PENDING_GPUS = max_plus_pending(MAX_GPUS, PENDING_GPUS)
 
+MAX_PERCENTAGE_EXPRESSION = (
+    "100"  # To help draw the max limit line on percentage panels
+)
 
 # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 # IMPORTANT: Please keep this in sync with Metrics.tsx and ray-metrics.rst
@@ -95,12 +98,12 @@ OVERVIEW_AND_HEALTH_PANELS = [
     Panel(
         id=44,
         title="Ray OOM Kills (Tasks and Actors)",
-        description="The number of tasks and actors killed by the Ray Out of Memory killer due to high memory pressure. Metrics are broken down by IP and the name. https://docs.ray.io/en/master/ray-core/scheduling/ray-oom-prevention.html.",
+        description="The number of tasks and actors killed by the Ray Out of Memory killer due to high memory pressure. Metrics are broken down by IP and the name. https://docs.ray.io/en/master/ray-core/scheduling/ray-oom-prevention.html. Note: The RayNodeType filter does not work on this graph.",
         unit="failures",
         targets=[
             Target(
-                expr='sum(ray_memory_manager_worker_eviction_total{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (Name, instance, RayNodeType)',
-                legend="OOM Killed: {{Name}}, {{instance}} ({{RayNodeType}})",
+                expr='sum(ray_memory_manager_worker_eviction_total{{instance=~"$Instance", {global_filters}}}) by (Name, instance)',
+                legend="OOM Killed: {{Name}}, {{instance}}",
             ),
         ],
     ),
@@ -209,12 +212,28 @@ RAY_TASKS_ACTORS_PLACEMENT_GROUPS_PANELS = [
             )
         ],
     ),
+    Panel(
+        id=29,
+        title="Object Store Memory by Location",
+        description="Object store memory usage by location. The dotted line indicates the object store memory capacity. This metric can go over the max capacity in case of spillage to disk.\n\nLocation: where the memory was allocated, which is MMAP_SHM or MMAP_DISK to indicate memory-mapped page, SPILLED to indicate spillage to disk, and WORKER_HEAP for objects small enough to be inlined in worker memory.",
+        unit="bytes",
+        targets=[
+            Target(
+                expr='sum(ray_object_store_memory{{instance=~"$Instance",{global_filters}}}) by (Location)',
+                legend="{{Location}}",
+            ),
+            Target(
+                expr='sum(ray_resources{{Name="object_store_memory",instance=~"$Instance",{global_filters}}})',
+                legend="MAX",
+            ),
+        ],
+    ),
 ]
 
 RAY_RESOURCES_PANELS = [
     Panel(
         id=27,
-        title="Logical CPUs used",
+        title="Logical CPUs Usage",
         description="Logical CPU usage of Ray. The dotted line indicates the total number of CPUs. The logical CPU is allocated by `num_cpus` arguments from tasks and actors. PENDING means the number of CPUs that will be available when new nodes are up after the autoscaler scales up.\n\nNOTE: Ray's logical CPU is different from physical CPU usage. Ray's logical CPU is allocated by `num_cpus` arguments.",
         unit="cores",
         targets=[
@@ -236,7 +255,7 @@ RAY_RESOURCES_PANELS = [
     ),
     Panel(
         id=28,
-        title="Logical GPUs used",
+        title="Logical GPUs Usage",
         description="Logical GPU usage of Ray. The dotted line indicates the total number of GPUs. The logical GPU is allocated by `num_gpus` arguments from tasks and actors. PENDING means the number of GPUs that will be available when new nodes are up after the autoscaler scales up.",
         unit="GPUs",
         targets=[
@@ -257,14 +276,14 @@ RAY_RESOURCES_PANELS = [
         ],
     ),
     Panel(
-        id=29,
-        title="Object Store Memory",
-        description="Object store memory usage by location. The dotted line indicates the object store memory capacity.\n\nLocation: where the memory was allocated, which is MMAP_SHM or MMAP_DISK to indicate memory-mapped page, SPILLED to indicate spillage to disk, and WORKER_HEAP for objects small enough to be inlined in worker memory. Refer to metric_defs.cc for more information.",
+        id=58,
+        title="Object Store Memory Usage",
+        description="Object store memory usage by instance, including memory that has been spilled to disk. The dotted line indicates the object store memory capacity. This metric can go over the max capacity in case of spillage to disk.",
         unit="bytes",
         targets=[
             Target(
-                expr='sum(ray_object_store_memory{{instance=~"$Instance",{global_filters}}}) by (Location)',
-                legend="{{Location}}",
+                expr='sum(ray_object_store_memory{{instance=~"$Instance",{global_filters}}}) by (instance)',
+                legend="{{instance}}",
             ),
             Target(
                 expr='sum(ray_resources{{Name="object_store_memory",instance=~"$Instance",{global_filters}}})',
@@ -272,12 +291,44 @@ RAY_RESOURCES_PANELS = [
             ),
         ],
     ),
+    Panel(
+        id=59,
+        title="Object Store Memory Usage %",
+        description="Object store memory usage % by instance, including memory that has been spilled to disk. This metric can go over 100% in case of spillage to disk.",
+        unit="%",
+        targets=[
+            Target(
+                expr='sum(ray_object_store_memory{{instance=~"$Instance",{global_filters}}}) by (instance) * 100 / sum(ray_resources{{Name="object_store_memory",instance=~"$Instance",{global_filters}}}) by (instance)',
+                legend="{{instance}}",
+            ),
+            Target(
+                expr=MAX_PERCENTAGE_EXPRESSION,  # To show the memory limit visually
+                legend="MAX",
+            ),
+        ],
+        fill=0,
+        stack=False,
+    ),
+    Panel(
+        id=60,
+        title="Object Store Memory Spilled to Disk",
+        description="Object store memory that has been spilled to disk, by instance.",
+        unit="bytes",
+        targets=[
+            Target(
+                expr='sum(ray_object_store_memory{{instance=~"$Instance",Location="SPILLED",{global_filters}}}) by (instance)',
+                legend="{{instance}}",
+            ),
+        ],
+        fill=0,
+        stack=False,
+    ),
 ]
 
 NODE_HARDWARE_UTILIZATION_BY_RAY_COMPONENT_PANELS = [
     Panel(
         id=37,
-        title="Node CPU by Component",
+        title="Node CPU Usage by Component",
         description="The physical (hardware) CPU usage across the cluster, broken down by component. This reports the summed CPU usage per Ray component. Ray components consist of system components (e.g., raylet, gcs, dashboard, or agent) and the process (that contains method names) names of running tasks/actors.",
         unit="cores",
         targets=[
@@ -294,7 +345,7 @@ NODE_HARDWARE_UTILIZATION_BY_RAY_COMPONENT_PANELS = [
     ),
     Panel(
         id=34,
-        title="Node Memory by Component",
+        title="Node Memory Usage by Component",
         description="The physical (hardware) memory usage across the cluster, broken down by component. This reports the summed RSS-SHM per Ray component, which corresponds to an approximate memory usage per proc. Ray components consist of system components (e.g., raylet, gcs, dashboard, or agent) and the process (that contains method names) names of running tasks/actors.",
         unit="bytes",
         targets=[
@@ -314,7 +365,7 @@ NODE_HARDWARE_UTILIZATION_BY_RAY_COMPONENT_PANELS = [
     ),
     Panel(
         id=45,
-        title="Node GPU by Component",
+        title="Node GPU Usage by Component",
         description="The physical (hardware) GPU usage across the cluster, broken down by component. This reports the summed GPU usage per Ray component.",
         unit="GPUs",
         targets=[
@@ -326,7 +377,7 @@ NODE_HARDWARE_UTILIZATION_BY_RAY_COMPONENT_PANELS = [
     ),
     Panel(
         id=46,
-        title="Node GPU Memory by Component",
+        title="Node GPU Memory Usage by Component",
         description="The physical (hardware) GPU memory usage across the cluster, broken down by component. This reports the summed GPU memory usage per Ray component.",
         unit="bytes",
         targets=[
@@ -345,8 +396,8 @@ NODE_HARDWARE_UTILIZATION_BY_RAY_COMPONENT_PANELS = [
 NODE_HARDWARE_UTILIZATION_PANELS = [
     Panel(
         id=2,
-        title="Node CPU utilization",
-        description="",
+        title="Node CPU Usage",
+        description="The physical (hardware) CPU usage for each node.",
         unit="cores",
         targets=[
             Target(
@@ -360,24 +411,22 @@ NODE_HARDWARE_UTILIZATION_PANELS = [
         ],
     ),
     Panel(
-        id=8,
-        title="Node GPU utilization",
-        description="Node's physical (hardware) GPU usage. The dotted line means the total number of hardware GPUs from the cluster. ",
-        unit="GPUs",
+        id=54,
+        title="Node CPU Usage %",
+        description="The percentage of physical (hardware) CPU usage for each node.",
+        unit="%",
         targets=[
             Target(
-                expr='sum(ray_node_gpus_utilization{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}} / 100) by (instance, RayNodeType, GpuIndex, GpuDeviceName)',
-                legend="GPU Usage: {{instance}} ({{RayNodeType}}), gpu.{{GpuIndex}}, {{GpuDeviceName}}",
-            ),
-            Target(
-                expr='sum(ray_node_gpus_available{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}})',
-                legend="MAX",
+                expr='sum(ray_node_cpu_utilization{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType)',
+                legend="CPU Usage: {{instance}} ({{RayNodeType}})",
             ),
         ],
+        fill=0,
+        stack=False,
     ),
     Panel(
         id=4,
-        title="Node Memory (heap + object store)",
+        title="Node Memory Usage (heap + object store)",
         description="The physical (hardware) memory usage for each node. The dotted line means the total amount of memory from the cluster. Node memory is a sum of object store memory (shared memory) and heap memory.\n\nNote: If Ray is deployed within a container, the total memory could be lower than the host machine because Ray may reserve some additional memory space outside the container.",
         unit="bytes",
         targets=[
@@ -393,21 +442,81 @@ NODE_HARDWARE_UTILIZATION_PANELS = [
     ),
     Panel(
         id=48,
-        title="Node Memory % (heap + object store)",
+        title="Node Memory Usage % (heap + object store)",
         description="The percentage of physical (hardware) memory usage for each node.",
         unit="%",
         targets=[
             Target(
-                expr='sum(ray_node_mem_used{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}/ray_node_mem_total{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}} * 100) by (instance, RayNodeType)',
+                expr='sum(ray_node_mem_used{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType) * 100 / sum(ray_node_mem_total{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType)',
                 legend="Memory Used: {{instance}} ({{RayNodeType}})",
             ),
         ],
         fill=0,
         stack=False,
     ),
+    Panel(
+        id=6,
+        title="Node Disk Usage",
+        description="Node's physical (hardware) disk usage. The dotted line means the total amount of disk space from the cluster.\n\nNOTE: When Ray is deployed within a container, this shows the disk usage from the host machine. ",
+        unit="bytes",
+        targets=[
+            Target(
+                expr='sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType)',
+                legend="Disk Used: {{instance}} ({{RayNodeType}})",
+            ),
+            Target(
+                expr='sum(ray_node_disk_free{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) + sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}})',
+                legend="MAX",
+            ),
+        ],
+    ),
+    Panel(
+        id=57,
+        title="Node Disk Usage %",
+        description="Node's physical (hardware) disk usage. \n\nNOTE: When Ray is deployed within a container, this shows the disk usage from the host machine. ",
+        unit="%",
+        targets=[
+            Target(
+                expr='sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType) * 100 / (sum(ray_node_disk_free{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType) + sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType))',
+                legend="Disk Used: {{instance}} ({{RayNodeType}})",
+            ),
+        ],
+        fill=0,
+        stack=False,
+    ),
+    Panel(
+        id=8,
+        title="Node GPU Usage",
+        description="Node's physical (hardware) GPU usage. The dotted line means the total number of hardware GPUs from the cluster. ",
+        unit="GPUs",
+        targets=[
+            Target(
+                expr='sum(ray_node_gpus_utilization{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}} / 100) by (instance, RayNodeType, GpuIndex, GpuDeviceName)',
+                legend="GPU Usage: {{instance}} ({{RayNodeType}}), gpu.{{GpuIndex}}, {{GpuDeviceName}}",
+            ),
+            Target(
+                expr='sum(ray_node_gpus_available{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}})',
+                legend="MAX",
+            ),
+        ],
+    ),
+    Panel(
+        id=55,
+        title="Node GPU Usage %",
+        description="Node's physical (hardware) GPU usage.",
+        unit="%",
+        targets=[
+            Target(
+                expr='sum(ray_node_gpus_utilization{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType, GpuIndex, GpuDeviceName)',
+                legend="GPU Usage: {{instance}} ({{RayNodeType}}), gpu.{{GpuIndex}}, {{GpuDeviceName}}",
+            ),
+        ],
+        fill=0,
+        stack=False,
+    ),
     Panel(
         id=18,
-        title="Node GPU Memory (GRAM)",
+        title="Node GPU Memory Usage (GRAM)",
         description="The physical (hardware) GPU memory usage for each node. The dotted line means the total amount of GPU memory from the cluster.",
         unit="bytes",
         targets=[
@@ -422,20 +531,18 @@ NODE_HARDWARE_UTILIZATION_PANELS = [
         ],
     ),
     Panel(
-        id=6,
-        title="Node Disk",
-        description="Node's physical (hardware) disk usage. The dotted line means the total amount of disk space from the cluster.\n\nNOTE: When Ray is deployed within a container, this shows the disk usage from the host machine. ",
-        unit="bytes",
+        id=56,
+        title="Node GPU Memory Usage (GRAM) %",
+        description="The percentage of physical (hardware) GPU memory usage for each node.",
+        unit="%",
         targets=[
             Target(
-                expr='sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType)',
-                legend="Disk Used: {{instance}} ({{RayNodeType}})",
-            ),
-            Target(
-                expr='sum(ray_node_disk_free{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) + sum(ray_node_disk_usage{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}})',
-                legend="MAX",
+                expr='sum(ray_node_gram_used{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType, GpuIndex, GpuDeviceName) * 100 / (sum(ray_node_gram_available{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType, GpuIndex, GpuDeviceName) + sum(ray_node_gram_used{{instance=~"$Instance", RayNodeType=~"$RayNodeType", {global_filters}}}) by (instance, RayNodeType, GpuIndex, GpuDeviceName))',
+                legend="Used GRAM: {{instance}} ({{RayNodeType}}), gpu.{{GpuIndex}}, {{GpuDeviceName}}",
             ),
         ],
+        fill=0,
+        stack=False,
     ),
     Panel(
         id=32,
@@ -534,15 +641,9 @@ DEFAULT_GRAFANA_ROWS = [
         collapsed=False,
     ),
     Row(
-        title="Ray Tasks, Actors and Placement Groups",
-        id=1002,
-        panels=RAY_TASKS_ACTORS_PLACEMENT_GROUPS_PANELS,
-        collapsed=False,
-    ),
-    Row(
-        title="Ray Resources",
-        id=1003,
-        panels=RAY_RESOURCES_PANELS,
+        title="Hardware Utilization by Node",
+        id=1005,
+        panels=NODE_HARDWARE_UTILIZATION_PANELS,
         collapsed=False,
     ),
     Row(
@@ -552,9 +653,15 @@ DEFAULT_GRAFANA_ROWS = [
         collapsed=False,
     ),
     Row(
-        title="Hardware Utilization by Node",
-        id=1005,
-        panels=NODE_HARDWARE_UTILIZATION_PANELS,
+        title="Ray Resources by Node",
+        id=1003,
+        panels=RAY_RESOURCES_PANELS,
+        collapsed=False,
+    ),
+    Row(
+        title="Ray Tasks, Actors and Placement Groups",
+        id=1002,
+        panels=RAY_TASKS_ACTORS_PLACEMENT_GROUPS_PANELS,
         collapsed=False,
     ),
     Row(
@@ -569,6 +676,9 @@ ids = []
 for row in DEFAULT_GRAFANA_ROWS:
     ids.append(row.id)
     ids.extend(panel.id for panel in row.panels)
+
+ids.sort()
+
 assert len(ids) == len(
     set(ids)
 ), f"Duplicated id found. Use unique id for each panel. {ids}"
