@@ -2,6 +2,10 @@ import logging
 import math
 from typing import TYPE_CHECKING, Optional
 
+from .actor_pool_resizing_policy import (
+    ActorPoolResizingPolicy,
+    DefaultResizingPolicy,
+)
 from .autoscaling_actor_pool import ActorPoolScalingRequest, AutoscalingActorPool
 from .base_actor_autoscaler import ActorAutoscaler
 from ray.data._internal.execution.interfaces.execution_options import ExecutionResources
@@ -22,6 +26,7 @@ class DefaultActorAutoscaler(ActorAutoscaler):
         resource_manager: "ResourceManager",
         *,
         config: AutoscalingConfig,
+        actor_pool_resizing_policy: Optional[ActorPoolResizingPolicy] = None,
     ):
         super().__init__(topology, resource_manager)
 
@@ -31,6 +36,15 @@ class DefaultActorAutoscaler(ActorAutoscaler):
         self._actor_pool_scaling_down_threshold = (
             config.actor_pool_util_downscaling_threshold
         )
+        self._actor_pool_max_upscaling_delta = config.actor_pool_max_upscaling_delta
+
+        self._actor_pool_resizing_policy = (
+            actor_pool_resizing_policy
+            or DefaultResizingPolicy(
+                upscaling_threshold=config.actor_pool_util_upscaling_threshold,
+                max_upscaling_delta=config.actor_pool_max_upscaling_delta,
+            )
+        )
 
         self._validate_autoscaling_config()
 
@@ -43,9 +57,47 @@ class DefaultActorAutoscaler(ActorAutoscaler):
                     self._derive_target_scaling_config(actor_pool, op, state)
                 )
 
+    def _compute_utilization(self, actor_pool: AutoscalingActorPool) -> float:
+        """Compute the utilization of the actor pool.
+
+        Args:
+            actor_pool: The actor pool to compute utilization for.
+
+        Returns:
+            The utilization value. Can be infinite if the actor pool has no running actors.
+        """
+        return actor_pool.get_pool_util()
+
+    def _compute_upscale_delta(
+        self,
+        actor_pool: AutoscalingActorPool,
+        util: float,
+    ) -> int:
+        """Compute how many actors to add when scaling up.
+
+        Args:
+            actor_pool: The actor pool to scale.
+            util: The current utilization of the actor pool.
+
+        Returns:
+            The number of actors to add (must be >= 1).
+        """
+        return self._actor_pool_resizing_policy.compute_upscale_delta(actor_pool, util)
+
+    def _compute_downscale_delta(self, actor_pool: AutoscalingActorPool) -> int:
+        """Compute how many actors to remove when scaling down.
+
+        Args:
+            actor_pool: The actor pool to scale down.
+
+        Returns:
+            The number of actors to remove (must be >= 1).
+        """
+        return self._actor_pool_resizing_policy.compute_downscale_delta(actor_pool)
+
     def _derive_target_scaling_config(
         self,
-        actor_pool: "AutoscalingActorPool",
+        actor_pool: AutoscalingActorPool,
         op: "PhysicalOperator",
         op_state: "OpState",
     ) -> ActorPoolScalingRequest:
@@ -53,8 +105,9 @@ class DefaultActorAutoscaler(ActorAutoscaler):
         if op.completed() or (
             op._inputs_complete and op_state.total_enqueued_input_blocks() == 0
         ):
+            num_to_scale_down = self._compute_downscale_delta(actor_pool)
             return ActorPoolScalingRequest.downscale(
-                delta=-1, force=True, reason="consumed all inputs"
+                delta=-num_to_scale_down, force=True, reason="consumed all inputs"
             )
 
         if actor_pool.current_size() < actor_pool.min_size():
@@ -64,22 +117,32 @@ class DefaultActorAutoscaler(ActorAutoscaler):
                 reason="pool below min size",
             )
         elif actor_pool.current_size() > actor_pool.max_size():
-            # Do not scale up, if the actor pool is already at max size.
             return ActorPoolScalingRequest.downscale(
-                # NOTE: For scale down delta has to be negative
                 delta=-(actor_pool.current_size() - actor_pool.max_size()),
                 reason="pool exceeding max size",
             )
 
         # Determine whether to scale up based on the actor pool utilization.
-        util = actor_pool.get_pool_util()
+        util = self._compute_utilization(actor_pool)
+
         if util >= self._actor_pool_scaling_up_threshold:
+            average_num_inputs_per_task = op.metrics.average_num_inputs_per_task or 1
+            # Skip scaling up if the current free task slots can handle all pending work.
+            # Each task consumes `average_num_inputs_per_task` input blocks on average,
+            # so the total input capacity is: free_slots × avg_inputs_per_task.
+            # If enqueued blocks ≤ capacity, we have enough resources already.
+            if (
+                op_state.total_enqueued_input_blocks()
+                <= actor_pool.num_free_task_slots() * average_num_inputs_per_task
+            ):
+                return ActorPoolScalingRequest.no_op(
+                    reason="enough free task slots to consume the existing inputs"
+                )
+
             # Do not scale up if either
             #   - Previous scale up has not finished yet
             #   - Actor Pool is at max size already
             #   - Op is throttled (ie exceeding allocated resource quota)
-            #   - Actor Pool has sufficient amount of slots available to handle
-            #   pending tasks
             if actor_pool.num_pending_actors() > 0:
                 return ActorPoolScalingRequest.no_op(reason="pending actors")
             elif actor_pool.current_size() >= actor_pool.max_size():
@@ -89,11 +152,20 @@ class DefaultActorAutoscaler(ActorAutoscaler):
                     reason="operator exceeding resource quota"
                 )
             budget = self._resource_manager.get_budget(op)
-            if _get_max_scale_up(actor_pool, budget) == 0:
+            max_scale_up = _get_max_scale_up(actor_pool, budget)
+            if max_scale_up == 0:
                 return ActorPoolScalingRequest.no_op(reason="exceeded resource limits")
+            if util == float("inf"):
+                return ActorPoolScalingRequest.upscale(
+                    delta=1, reason="no running actors, scale up immediately"
+                )
+            delta = self._compute_upscale_delta(actor_pool, util)
+            if max_scale_up is not None:
+                delta = min(delta, max_scale_up)
+            delta = max(1, delta)  # At least scale up by 1
 
             return ActorPoolScalingRequest.upscale(
-                delta=1,
+                delta=delta,
                 reason=(
                     f"utilization of {util} >= "
                     f"{self._actor_pool_scaling_up_threshold}"
@@ -103,8 +175,13 @@ class DefaultActorAutoscaler(ActorAutoscaler):
             if actor_pool.current_size() <= actor_pool.min_size():
                 return ActorPoolScalingRequest.no_op(reason="reached min size")
 
+            max_can_release = actor_pool.current_size() - actor_pool.min_size()
+            num_to_scale_down = min(
+                self._compute_downscale_delta(actor_pool), max_can_release
+            )
+
             return ActorPoolScalingRequest.downscale(
-                delta=-1,
+                delta=-num_to_scale_down,
                 reason=(
                     f"utilization of {util} <= "
                     f"{self._actor_pool_scaling_down_threshold}"
@@ -120,13 +197,27 @@ class DefaultActorAutoscaler(ActorAutoscaler):
             )
 
     def _validate_autoscaling_config(self):
+        # Validate that max upscaling delta is positive to prevent override by safeguard
+        if self._actor_pool_max_upscaling_delta <= 0:
+            raise ValueError(
+                f"actor_pool_max_upscaling_delta must be positive, "
+                f"got {self._actor_pool_max_upscaling_delta}"
+            )
+        # Validate that upscaling threshold is positive to prevent division by zero
+        # and incorrect scaling calculations
+        if self._actor_pool_scaling_up_threshold <= 0:
+            raise ValueError(
+                f"actor_pool_util_upscaling_threshold must be positive, "
+                f"got {self._actor_pool_scaling_up_threshold}"
+            )
+
         for op, state in self._topology.items():
             for actor_pool in op.get_autoscaling_actor_pools():
                 self._validate_actor_pool_autoscaling_config(actor_pool, op)
 
     def _validate_actor_pool_autoscaling_config(
         self,
-        actor_pool: "AutoscalingActorPool",
+        actor_pool: AutoscalingActorPool,
         op: "PhysicalOperator",
     ) -> None:
         """Validate autoscaling configuration.
