@@ -135,8 +135,8 @@ class ResourceManager:
             ):
                 logger.warning(
                     f"{WARN_PREFIX} Ray's object store is configured to use only "
-                    f"{object_store_fraction:.1%} of available memory ({object_store_memory/GiB:.1f}GiB "
-                    f"out of {total_memory/GiB:.1f}GiB total). For optimal Ray Data performance, "
+                    f"{object_store_fraction:.1%} of available memory ({object_store_memory / GiB:.1f}GiB "
+                    f"out of {total_memory / GiB:.1f}GiB total). For optimal Ray Data performance, "
                     f"we recommend setting the object store to at least 50% of available memory. "
                     f"You can do this by setting the 'object_store_memory' parameter when calling "
                     f"ray.init() or by setting the RAY_DEFAULT_OBJECT_STORE_MEMORY_PROPORTION environment variable."
@@ -268,15 +268,25 @@ class ResourceManager:
         """Return the resource usage of the given operator at the current time."""
         return self._op_usages[op]
 
+    def get_mem_op_internal(self, op: PhysicalOperator) -> int:
+        """Return the memory usage of the internal buffers of the given operator."""
+        return self._mem_op_internal[op]
+
+    def get_mem_op_outputs(self, op: PhysicalOperator) -> int:
+        """Return the memory usage of the outputs of the given operator."""
+        return self._mem_op_outputs[op]
+
     def get_op_usage_str(self, op: PhysicalOperator, *, verbose: bool) -> str:
         """Return a human-readable string representation of the resource usage of
         the given operator."""
-        usage_str = f"{self._op_running_usages[op].cpu:.1f} CPU"
-        if self._op_running_usages[op].gpu:
-            usage_str += f", {self._op_running_usages[op].gpu:.1f} GPU"
-        usage_str += (
-            f", {self._op_running_usages[op].object_store_memory_str()} object store"
-        )
+        # Handle case where operator is not in _op_running_usages dict
+        if op not in self._op_running_usages:
+            usage_str = "n/a"
+        else:
+            usage_str = f"{self._op_running_usages[op].cpu:.1f} CPU"
+            if self._op_running_usages[op].gpu:
+                usage_str += f", {self._op_running_usages[op].gpu:.1f} GPU"
+            usage_str += f", {self._op_running_usages[op].object_store_memory_str()} object store"
 
         # NOTE: Config can override requested verbosity level
         if LOG_DEBUG_TELEMETRY_FOR_RESOURCE_MANAGER_OVERRIDE is not None:
@@ -284,8 +294,8 @@ class ResourceManager:
 
         if verbose:
             usage_str += (
-                f" (in={memory_string(self._mem_op_internal[op])},"
-                f"out={memory_string(self._mem_op_outputs[op])})"
+                f" (in={memory_string(self.get_mem_op_internal(op))},"
+                f"out={memory_string(self.get_mem_op_outputs(op))})"
             )
             if self._op_resource_allocator is not None:
                 allocation = self._op_resource_allocator.get_allocation(op)
@@ -341,7 +351,7 @@ class ResourceManager:
             not op.throttling_disabled()
             # As long as the op has finished execution, even if there are still
             # non-taken outputs, we don't need to allocate resources for it.
-            and not op.execution_finished()
+            and not op.has_execution_finished()
         )
 
     def get_eligible_ops(self) -> List[PhysicalOperator]:
@@ -392,9 +402,12 @@ class ResourceManager:
         )
         return op_outputs_usage
 
-    def get_op_internal_object_store_usage(self, op: PhysicalOperator) -> int:
-        """Get the internal object store memory usage of the given operator"""
-        return self._mem_op_internal[op]
+    def has_materializing_downstream_op(self, op: PhysicalOperator) -> bool:
+        """Check if the operator has a downstream materializing operator."""
+        return any(
+            isinstance(next_op, MATERIALIZING_OPERATORS)
+            for next_op in op.output_dependencies
+        )
 
 
 def _get_first_pending_shuffle_op(topology: "Topology") -> int:
@@ -553,7 +566,7 @@ class OpResourceAllocator(ABC):
             not op.throttling_disabled()
             # As long as the op has finished execution, even if there are still
             # non-taken outputs, we don't need to allocate resources for it.
-            and not op.execution_finished()
+            and not op.has_execution_finished()
         )
 
     def _get_downstream_eligible_ops(
@@ -674,9 +687,9 @@ class ReservationOpResourceAllocator(OpResourceAllocator):
         ops_to_exclude_from_reservation = []
         # Traverse operator tree collecting all operators that have already finished
         for op in self._topology:
-            if not op.execution_finished():
+            if not op.has_execution_finished():
                 for dep in op.input_dependencies:
-                    if dep.execution_finished():
+                    if dep.has_execution_finished():
                         last_completed_ops.append(dep)
 
         # In addition to completed operators,
@@ -830,9 +843,7 @@ class ReservationOpResourceAllocator(OpResourceAllocator):
             op_mem_usage = 0
             # Add the memory usage of the operator itself,
             # excluding `_reserved_for_op_outputs`.
-            op_mem_usage += self._resource_manager.get_op_internal_object_store_usage(
-                op
-            )
+            op_mem_usage += self._resource_manager.get_mem_op_internal(op)
             # Add the portion of op outputs usage that has
             # exceeded `_reserved_for_op_outputs`.
             op_outputs_usage = self._resource_manager.get_op_outputs_object_store_usage_with_downstream(
