@@ -11,8 +11,10 @@ from ray.data._internal.issue_detection.issue_detector import (
 )
 
 if TYPE_CHECKING:
+    from ray.data._internal.execution.interfaces.physical_operator import (
+        PhysicalOperator,
+    )
     from ray.data._internal.execution.streaming_executor import StreamingExecutor
-    from ray.data.context import DataContext
 
 HIGH_MEMORY_PERIODIC_WARNING = """
 Operator '{op_name}' uses {memory_per_task} of memory per task on average, but Ray
@@ -35,25 +37,48 @@ class HighMemoryIssueDetectorConfig:
 
 
 class HighMemoryIssueDetector(IssueDetector):
-
     # Many nodes have a 4 GiB : 1 core ratio, but this isn't always the case (e.g., for
     # high memory nodes).
     _MEMORY_PER_CORE_ESTIMATE = 4 * 1024**3
 
-    def __init__(self, executor: "StreamingExecutor", ctx: "DataContext"):
-        super().__init__(executor, ctx)
-        self._detector_cfg = ctx.issue_detectors_config.high_memory_detector_config
+    def __init__(
+        self,
+        dataset_id: str,
+        operators: List["PhysicalOperator"],
+        config: HighMemoryIssueDetectorConfig,
+    ):
+        self._dataset_id = dataset_id
+        self._detector_cfg = config
+        self._operators = operators
 
         self._initial_memory_requests: Dict[MapOperator, int] = {}
-        for op in self._executor._topology.keys():
+        for op in operators:
             if isinstance(op, MapOperator):
                 self._initial_memory_requests[op] = (
                     op._get_dynamic_ray_remote_args().get("memory") or 0
                 )
 
+    @classmethod
+    def from_executor(cls, executor: "StreamingExecutor") -> "HighMemoryIssueDetector":
+        """Factory method to create a HighMemoryIssueDetector from a StreamingExecutor.
+
+        Args:
+            executor: The StreamingExecutor instance to extract dependencies from.
+
+        Returns:
+            An instance of HighMemoryIssueDetector.
+        """
+        operators = list(executor._topology.keys()) if executor._topology else []
+        ctx = executor._data_context
+        return cls(
+            dataset_id=executor._dataset_id,
+            operators=operators,
+            config=ctx.issue_detectors_config.high_memory_detector_config,
+        )
+
     def detect(self) -> List[Issue]:
         issues = []
-        for op in self._executor._topology.keys():
+        for op in self._operators:
             if not isinstance(op, MapOperator):
                 continue
 
@@ -78,7 +103,7 @@ class HighMemoryIssueDetector(IssueDetector):
                 )
                 issues.append(
                     Issue(
-                        dataset_name=self._executor._dataset_id,
+                        dataset_name=self._dataset_id,
                         operator_id=op.id,
                         issue_type=IssueType.HIGH_MEMORY,
                         message=_format_message(message),
