@@ -277,8 +277,19 @@ class Filter(AbstractUDFMap):
 
     def _get_operator_name(self, op_name: str, fn: UserDefinedFunction):
         if self.is_expression_based():
-            # TODO: Use a truncated expression prefix here instead of <expression>.
-            return f"{op_name}(<expression>)"
+            # Get a concise inline string representation of the expression
+            from ray.data._internal.planner.plan_expression.expression_visitors import (
+                _InlineExprReprVisitor,
+            )
+
+            expr_str = _InlineExprReprVisitor().visit(self._predicate_expr)
+
+            # Truncate only the final result if too long
+            max_length = 60
+            if len(expr_str) > max_length:
+                expr_str = expr_str[: max_length - 3] + "..."
+
+            return f"{op_name}({expr_str})"
         return super()._get_operator_name(op_name, fn)
 
 
@@ -390,7 +401,10 @@ class StreamingRepartition(AbstractMap):
         input_op: LogicalOperator,
         target_num_rows_per_block: int,
     ):
-        super().__init__("StreamingRepartition", input_op)
+        super().__init__(
+            f"StreamingRepartition[num_rows_per_block={target_num_rows_per_block}]",
+            input_op,
+        )
         self._target_num_rows_per_block = target_num_rows_per_block
 
     @property
