@@ -1,6 +1,6 @@
 import itertools
 import logging
-from typing import List, Optional
+from typing import Any, Dict, List, Optional
 
 from ray.data._internal.compute import (
     ActorPoolStrategy,
@@ -34,8 +34,11 @@ from ray.data._internal.logical.operators.all_to_all_operator import (
 from ray.data._internal.logical.operators.map_operator import (
     AbstractMap,
     AbstractUDFMap,
+    MapBatches,
+    StreamingRepartition,
 )
-from ray.data.context import DataContext
+from ray.data._internal.streaming_repartition import StreamingRepartitionRefBundler
+from ray.util.annotations import DeveloperAPI
 
 # Scheduling strategy can be inherited from upstream operator if not specified.
 INHERITABLE_REMOTE_ARGS = ["scheduling_strategy"]
@@ -49,9 +52,13 @@ class FuseOperators(Rule):
 
     def apply(self, plan: PhysicalPlan) -> PhysicalPlan:
         self._op_map = plan.op_map.copy()
+        # TODO(xgui): Currently we have to fuse streaming_repartition before map fusion
+        # because the result of map fusion loses the batch_size information.
+        # We should fix this by not losing the batch_size information when fusing map operators.
+        fused_dag = self._fuse_streaming_repartition_operators_in_dag(plan.dag)
         # Do DFS fusion on compatible pairwise operators in two passes.
         # In the first pass, only fuse back-to-back map operators together.
-        fused_dag = self._fuse_map_operators_in_dag(plan.dag)
+        fused_dag = self._fuse_map_operators_in_dag(fused_dag)
 
         # Now that we have fused together all back-to-back map operators,
         # we fuse together MapOperator -> AllToAllOperator pairs.
@@ -76,6 +83,30 @@ class FuseOperators(Rule):
             input._output_dependencies.append(op)
             self._update_output_deps(input)
 
+    def _fuse_streaming_repartition_operators_in_dag(
+        self, dag: PhysicalOperator
+    ) -> PhysicalOperator:
+        """Fuse (MapBatches -> StreamingRepartition) pair.
+
+        This will ensure the map_batch's function receive the correct number of rows.
+        We also ensure the output rows is `batch_size`.
+        """
+        upstream_ops = dag.input_dependencies
+        while (
+            len(upstream_ops) == 1
+            and isinstance(self._op_map[dag], StreamingRepartition)
+            and isinstance(self._op_map[upstream_ops[0]], MapBatches)
+            and self._can_fuse(dag, upstream_ops[0])
+        ):
+            dag = self._get_fused_streaming_repartition_operator(dag, upstream_ops[0])
+            upstream_ops = dag.input_dependencies
+
+        dag._input_dependencies = [
+            self._fuse_streaming_repartition_operators_in_dag(upstream_op)
+            for upstream_op in upstream_ops
+        ]
+        return dag
+
     def _fuse_map_operators_in_dag(self, dag: PhysicalOperator) -> MapOperator:
         """Starting at the given operator, traverses up the DAG of operators
         and recursively fuses compatible MapOperator -> MapOperator pairs.
@@ -198,7 +229,7 @@ class FuseOperators(Rule):
             return False
 
         # Only fuse if the ops' remote arguments are compatible.
-        if not _are_remote_args_compatible(
+        if not are_remote_args_compatible(
             getattr(up_logical_op, "_ray_remote_args", {}),
             getattr(down_logical_op, "_ray_remote_args", {}),
         ):
@@ -212,15 +243,95 @@ class FuseOperators(Rule):
             return False
 
         if not self._can_merge_target_max_block_size(
-            up_op.target_max_block_size,
-            down_op.target_max_block_size,
-            up_op.data_context,
+            up_op.target_max_block_size_override,
+            down_op.target_max_block_size_override,
         ):
             return False
 
+        # only allow fusion of MapBatches -> StreamingRepartition
+        if isinstance(down_logical_op, StreamingRepartition):
+            return (
+                isinstance(up_logical_op, MapBatches)
+                and up_logical_op._batch_size
+                == down_logical_op.target_num_rows_per_block
+            )
+        # Other operators cannot fuse with StreamingRepartition.
+        if isinstance(up_logical_op, StreamingRepartition):
+            return False
+
         # Otherwise, ops are compatible for fusion.
         return True
 
+    def _get_fused_streaming_repartition_operator(
+        self, down_op: PhysicalOperator, up_op: PhysicalOperator
+    ) -> PhysicalOperator:
+        assert self._can_fuse(down_op, up_op), (
+            "Current rule supports fusing MapBatches->StreamingRepartition, but received: "
+            f"{type(up_op).__name__} -> {type(down_op).__name__}"
+        )
+
+        name = up_op.name + "->" + down_op.name
+
+        down_logical_op = self._op_map.pop(down_op)
+        up_logical_op = self._op_map.pop(up_op)
+        assert isinstance(up_logical_op, MapBatches)
+        assert isinstance(down_logical_op, StreamingRepartition)
+        assert up_logical_op._batch_size == down_logical_op.target_num_rows_per_block
+        batch_size = up_logical_op._batch_size
+
+        compute = self._fuse_compute_strategy(
+            up_logical_op._compute, down_logical_op._compute
+        )
+        assert compute is not None
+
+        map_task_kwargs = {**up_op._map_task_kwargs, **down_op._map_task_kwargs}
+
+        ray_remote_args = up_logical_op._ray_remote_args
+        ray_remote_args_fn = (
+            up_logical_op._ray_remote_args_fn or down_logical_op._ray_remote_args_fn
+        )
+        input_deps = up_op.input_dependencies
+        assert len(input_deps) == 1
+        input_op = input_deps[0]
+
+        assert up_op.data_context is down_op.data_context
+        op = MapOperator.create(
+            up_op.get_map_transformer().fuse(down_op.get_map_transformer()),
+            input_op,
+            up_op.data_context,
+            name=name,
+            compute_strategy=compute,
+            ref_bundler=StreamingRepartitionRefBundler(batch_size),
+            map_task_kwargs=map_task_kwargs,
+            ray_remote_args=ray_remote_args,
+            ray_remote_args_fn=ray_remote_args_fn,
+            # For now, we don't want to over-fuse StreamingRepartition with other map operators,
+            # so the result operator does not support further fusion.
+            supports_fusion=False,
+        )
+        op.set_logical_operators(*up_op._logical_operators, *down_op._logical_operators)
+        for map_task_kwargs_fn in itertools.chain(
+            up_op._map_task_kwargs_fns, down_op._map_task_kwargs_fns
+        ):
+            op.add_map_task_kwargs_fn(map_task_kwargs_fn)
+
+        input_op = up_logical_op.input_dependency
+        logical_op = AbstractUDFMap(
+            name,
+            input_op,
+            up_logical_op._fn,
+            fn_args=up_logical_op._fn_args,
+            fn_kwargs=up_logical_op._fn_kwargs,
+            fn_constructor_args=up_logical_op._fn_constructor_args,
+            fn_constructor_kwargs=up_logical_op._fn_constructor_kwargs,
+            min_rows_per_bundled_input=batch_size,
+            compute=compute,
+            ray_remote_args_fn=ray_remote_args_fn,
+            ray_remote_args=ray_remote_args,
+        )
+        self._op_map[op] = logical_op
+        return op
+
     @classmethod
     def _fuse_compute_strategy(
         cls, up_compute: ComputeStrategy, down_compute: ComputeStrategy
@@ -250,35 +361,27 @@ class FuseOperators(Rule):
         self,
         up_target_max_block_size: Optional[int],
         down_target_max_block_size: Optional[int],
-        data_context: DataContext,
-    ):
-        # If the upstream op overrode the target max block size, only fuse if
-        # they are equal.
-        if up_target_max_block_size is not None:
-            if down_target_max_block_size is None:
-                down_target_max_block_size = data_context.target_max_block_size
-            if up_target_max_block_size != down_target_max_block_size:
-                return False
+    ) -> bool:
+        if (
+            up_target_max_block_size is not None
+            and down_target_max_block_size is not None
+        ):
+            # NOTE: In case of both ops overriding `target_max_block_size` only
+            #       merge them if settings are equal
+            return down_target_max_block_size == up_target_max_block_size
+
         return True
 
     def _get_merged_target_max_block_size(
         self,
         up_target_max_block_size: Optional[int],
         down_target_max_block_size: Optional[int],
-    ):
-        if up_target_max_block_size is not None:
-            # If the upstream op overrode the target max block size, we can
-            # only merge if the downstream op matches or uses the default.
-            assert (
-                down_target_max_block_size is None
-                or down_target_max_block_size == up_target_max_block_size
-            )
-            return up_target_max_block_size
-        else:
-            # Upstream op inherits the downstream op's target max block size,
-            # because the downstream op is the one that outputs the final
-            # blocks.
-            return down_target_max_block_size
+    ) -> Optional[int]:
+        assert self._can_merge_target_max_block_size(
+            up_target_max_block_size, down_target_max_block_size
+        )
+
+        return up_target_max_block_size or down_target_max_block_size
 
     def _get_fused_map_operator(
         self, down_op: MapOperator, up_op: MapOperator
@@ -302,7 +405,7 @@ class FuseOperators(Rule):
         )
 
         target_max_block_size = self._get_merged_target_max_block_size(
-            up_op.target_max_block_size, down_op.target_max_block_size
+            up_op.target_max_block_size_override, down_op.target_max_block_size_override
         )
 
         compute = self._fuse_compute_strategy(
@@ -322,19 +425,34 @@ class FuseOperators(Rule):
         assert len(input_deps) == 1
         input_op = input_deps[0]
 
+        # Fuse on_start callbacks from both operators.
+        # This preserves deferred initialization (e.g., on_write_start for Write ops).
+        up_on_start = up_op._on_start
+        down_on_start = down_op._on_start
+        if up_on_start is not None and down_on_start is not None:
+
+            def fused_on_start(schema):
+                up_on_start(schema)
+                down_on_start(schema)
+
+            on_start = fused_on_start
+        else:
+            on_start = up_on_start or down_on_start
+
         # Fused physical map operator.
         assert up_op.data_context is down_op.data_context
         op = MapOperator.create(
             up_op.get_map_transformer().fuse(down_op.get_map_transformer()),
             input_op,
             up_op.data_context,
-            target_max_block_size=target_max_block_size,
+            target_max_block_size_override=target_max_block_size,
             name=name,
             compute_strategy=compute,
             min_rows_per_bundle=min_rows_per_bundled_input,
             map_task_kwargs=map_task_kwargs,
             ray_remote_args=ray_remote_args,
             ray_remote_args_fn=ray_remote_args_fn,
+            on_start=on_start,
         )
         op.set_logical_operators(*up_op._logical_operators, *down_op._logical_operators)
         for map_task_kwargs_fn in itertools.chain(
@@ -436,7 +554,7 @@ class FuseOperators(Rule):
         input_op = input_deps[0]
 
         target_max_block_size = self._get_merged_target_max_block_size(
-            up_op.target_max_block_size, down_op.target_max_block_size
+            up_op.target_max_block_size_override, down_op.target_max_block_size_override
         )
 
         assert up_op.data_context is down_op.data_context
@@ -444,7 +562,7 @@ class FuseOperators(Rule):
             fused_all_to_all_transform_fn,
             input_op,
             up_op.data_context,
-            target_max_block_size=target_max_block_size,
+            target_max_block_size_override=target_max_block_size,
             num_outputs=down_op._num_outputs,
             # Transfer over the existing sub-progress bars from
             # the AllToAllOperator (if any) into the fused operator.
@@ -515,7 +633,10 @@ class FuseOperators(Rule):
         return True
 
 
-def _are_remote_args_compatible(prev_args, next_args):
+@DeveloperAPI
+def are_remote_args_compatible(
+    prev_args: Dict[str, Any], next_args: Dict[str, Any]
+) -> bool:
     """Check if Ray remote arguments are compatible for merging."""
     prev_args = _canonicalize(prev_args)
     next_args = _canonicalize(next_args)
