@@ -8,6 +8,7 @@ from ray.data.expressions import (
     DownloadExpr,
     Expr,
     LiteralExpr,
+    Operation,
     StarExpr,
     UDFExpr,
     UnaryExpr,
@@ -16,6 +17,25 @@ from ray.data.expressions import (
 
 T = TypeVar("T")
 
+# Mapping of operations to their string symbols for inline representation
+_INLINE_OP_SYMBOLS = {
+    Operation.ADD: "+",
+    Operation.SUB: "-",
+    Operation.MUL: "*",
+    Operation.DIV: "/",
+    Operation.FLOORDIV: "//",
+    Operation.GT: ">",
+    Operation.LT: "<",
+    Operation.GE: ">=",
+    Operation.LE: "<=",
+    Operation.EQ: "==",
+    Operation.NE: "!=",
+    Operation.AND: "&",
+    Operation.OR: "|",
+    Operation.IN: "in",
+    Operation.NOT_IN: "not in",
+}
+
 
 class _ExprVisitorBase(_ExprVisitor[None]):
     """Base visitor that provides automatic recursive traversal.
@@ -347,3 +367,115 @@ class _TreeReprVisitor(_ExprVisitor[str]):
 
     def visit_star(self, expr: "StarExpr") -> str:
         return self._make_tree_lines("COL(*)", expr=expr)
+
+
+class _InlineExprReprVisitor(_ExprVisitor[str]):
+    """Visitor that generates concise inline string representations of expressions.
+
+    This visitor creates single-line string representations suitable for displaying
+    in operator names, log messages, etc. It aims to be human-readable while keeping
+    the representation compact.
+    """
+
+    def __init__(self, max_literal_length: int = 20):
+        """Initialize the inline representation visitor.
+
+        Args:
+            max_literal_length: Maximum length for literal value representations
+        """
+        self._max_literal_length = max_literal_length
+
+    def visit_column(self, expr: "ColumnExpr") -> str:
+        """Visit a column expression and return its inline representation."""
+        return f"col({expr.name!r})"
+
+    def visit_literal(self, expr: "LiteralExpr") -> str:
+        """Visit a literal expression and return its inline representation."""
+        value_repr = repr(expr.value)
+        if len(value_repr) > self._max_literal_length:
+            value_repr = value_repr[: self._max_literal_length - 3] + "..."
+        return value_repr
+
+    def visit_binary(self, expr: "BinaryExpr") -> str:
+        """Visit a binary expression and return its inline representation."""
+        left_str = self.visit(expr.left)
+        right_str = self.visit(expr.right)
+
+        # Add parentheses around child binary expressions to avoid ambiguity
+        if isinstance(expr.left, BinaryExpr):
+            left_str = f"({left_str})"
+        if isinstance(expr.right, BinaryExpr):
+            right_str = f"({right_str})"
+
+        op_str = _INLINE_OP_SYMBOLS.get(expr.op, expr.op.name.lower())
+        return f"{left_str} {op_str} {right_str}"
+
+    def visit_unary(self, expr: "UnaryExpr") -> str:
+        """Visit a unary expression and return its inline representation."""
+        operand_str = self.visit(expr.operand)
+
+        # Add parentheses around binary expression operands to avoid ambiguity
+        if isinstance(expr.operand, BinaryExpr):
+            operand_str = f"({operand_str})"
+
+        # Map operations to symbols/functions
+        if expr.op == Operation.NOT:
+            return f"~{operand_str}"
+        elif expr.op == Operation.IS_NULL:
+            return f"{operand_str}.is_null()"
+        elif expr.op == Operation.IS_NOT_NULL:
+            return f"{operand_str}.is_not_null()"
+        else:
+            return f"{expr.op.name.lower()}({operand_str})"
+
+    def visit_alias(self, expr: "AliasExpr") -> str:
+        """Visit an alias expression and return its inline representation."""
+        inner_str = self.visit(expr.expr)
+        return f"{inner_str}.alias({expr.name!r})"
+
+    def visit_udf(self, expr: "UDFExpr") -> str:
+        """Visit a UDF expression and return its inline representation."""
+        # Get function name for better readability
+        # For callable objects (instances with __call__), use the class name
+        fn_name = getattr(expr.fn, "__name__", expr.fn.__class__.__name__)
+
+        # Build argument list
+        args_str = []
+        for arg in expr.args:
+            args_str.append(self.visit(arg))
+        for key, value in expr.kwargs.items():
+            args_str.append(f"{key}={self.visit(value)}")
+
+        args_repr = ", ".join(args_str) if args_str else ""
+        return f"{fn_name}({args_repr})"
+
+    def visit_download(self, expr: "DownloadExpr") -> str:
+        """Visit a download expression and return its inline representation."""
+        return f"download({expr.uri_column_name!r})"
+
+    def visit_star(self, expr: "StarExpr") -> str:
+        """Visit a star expression and return its inline representation."""
+        return "col(*)"
+
+
+def get_column_references(expr: Expr) -> List[str]:
+    """Extract all column references from an expression.
+
+    This is a convenience function that creates a _ColumnReferenceCollector,
+    visits the expression tree, and returns the list of referenced column names.
+
+    Args:
+        expr: The expression to extract column references from.
+
+    Returns:
+        List of column names referenced in the expression, in order of appearance.
+
+    Example:
+        >>> from ray.data.expressions import col
+        >>> expr = (col("a") > 5) & (col("b") == "test")
+        >>> get_column_references(expr)
+        ['a', 'b']
+    """
+    collector = _ColumnReferenceCollector()
+    collector.visit(expr)
+    return collector.get_column_refs()
