@@ -1,5 +1,7 @@
 import logging
+import sys
 import threading
+import time
 from abc import ABC, abstractmethod
 from typing import Any, List, Optional
 
@@ -131,14 +133,29 @@ class ProgressBar(AbstractProgressBar):
     ):
         self._desc = truncate_operator_name(name, self.MAX_NAME_LENGTH)
         self._progress = 0
+        self._total = total
         # Prepend a space to the unit for better formatting.
         if unit[0] != " ":
             unit = " " + unit
 
+        self._use_logging = False
+
         if enabled is None:
+            # When enabled is None (not explicitly set by the user),
+            # check DataContext setting
             from ray.data.context import DataContext
 
             enabled = DataContext.get_current().enable_progress_bars
+
+        # If enabled, and in non-interactive terminal, use logging instead of tqdm
+        if enabled and not sys.stdout.isatty():
+            self._use_logging = True
+            enabled = False
+            if log_once("progress_bar_disabled"):
+                logger.info(
+                    "Progress bar disabled because stdout is a non-interactive terminal."
+                )
+
         if not enabled:
             self._bar = None
         elif tqdm:
@@ -163,8 +180,18 @@ class ProgressBar(AbstractProgressBar):
                 needs_warning = False
             self._bar = None
 
+        # For logging progress in non-interactive terminals
+        self._last_logged_time = 0
+        # Log interval in seconds
+        from ray.data.context import DataContext
+
+        self._log_interval = DataContext.get_current().progress_bar_log_interval
+        self._logged_once = False
+
     def set_description(self, name: str) -> None:
         name = truncate_operator_name(name, self.MAX_NAME_LENGTH)
+        if self._use_logging:
+            self._desc = name
         if self._bar and name != self._desc:
             self._desc = name
             self._bar.set_description(self._desc)
@@ -172,9 +199,30 @@ class ProgressBar(AbstractProgressBar):
     def get_description(self) -> str:
         return self._desc
 
+    def _log_progress_if_needed(self):
+        """Log progress if the required time interval has passed."""
+        current_time = time.time()
+        time_diff = current_time - self._last_logged_time
+        should_log = (self._last_logged_time == 0) or (time_diff >= self._log_interval)
+
+        if should_log:
+            # Remove leading hyphens from the description
+            clean_desc = self._desc.lstrip("- ").strip()
+            if not self._logged_once:
+                operation_name = clean_desc.split(":")[0]
+                logger.info(f"=== Ray Data Progress {{{operation_name}}} ===")
+                self._logged_once = True
+            logger.info(
+                f"{clean_desc}: Progress Completed {self._progress} / {self._total or '?'}"
+            )
+            self._last_logged_time = current_time
+
     def refresh(self):
         if self._bar:
             self._bar.refresh()
+        elif self._use_logging:
+            # Log progress periodically
+            self._log_progress_if_needed()
 
     def update(self, increment: int = 0, total: Optional[int] = None) -> None:
         if self._bar and (increment != 0 or self._bar.total != total):
@@ -185,6 +233,13 @@ class ProgressBar(AbstractProgressBar):
                 # If the progress goes over 100%, update the total.
                 self._bar.total = self._progress
             self._bar.update(increment)
+        elif self._use_logging and (increment is not None and increment != 0):
+            self._progress += increment
+            if total is not None:
+                self._total = total
+
+            # Log progress periodically
+            self._log_progress_if_needed()
 
     def close(self):
         if self._bar:
