@@ -1,5 +1,7 @@
+import logging
 from typing import Any, Dict, Optional
 
+from ray._common.deprecation import Deprecated
 from ray.data.block import UserDefinedFunction
 from ray.llm._internal.batch.processor import (
     HttpRequestProcessorConfig as _HttpRequestProcessorConfig,
@@ -9,8 +11,17 @@ from ray.llm._internal.batch.processor import (
     SGLangEngineProcessorConfig as _SGLangEngineProcessorConfig,
     vLLMEngineProcessorConfig as _vLLMEngineProcessorConfig,
 )
+from ray.llm._internal.batch.stages.configs import (
+    ChatTemplateStageConfig as _ChatTemplateStageConfig,
+    DetokenizeStageConfig as _DetokenizeStageConfig,
+    PrepareImageStageConfig as _PrepareImageStageConfig,
+    PrepareMultimodalStageConfig as _PrepareMultimodalStageConfig,
+    TokenizerStageConfig as _TokenizerStageConfig,
+)
 from ray.util.annotations import PublicAPI
 
+logger = logging.getLogger(__name__)
+
 
 @PublicAPI(stability="alpha")
 class ProcessorConfig(_ProcessorConfig):
@@ -55,14 +66,14 @@ class HttpRequestProcessorConfig(_HttpRequestProcessorConfig):
             :skipif: True
 
             import ray
-            from ray.data.llm import HttpRequestProcessorConfig, build_llm_processor
+            from ray.data.llm import HttpRequestProcessorConfig, build_processor
 
             config = HttpRequestProcessorConfig(
                 url="https://api.openai.com/v1/chat/completions",
                 headers={"Authorization": "Bearer sk-..."},
                 concurrency=1,
             )
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     payload=dict(
@@ -113,13 +124,28 @@ class vLLMEngineProcessorConfig(_vLLMEngineProcessorConfig):
             each batch. The default value may not be optimal when the batch size
             or the batch processing latency is too small, but it should be good
             enough for batch size >= 64.
-        apply_chat_template: Whether to apply chat template.
-        chat_template: The chat template to use. This is usually not needed if the
-            model checkpoint already contains the chat template.
-        tokenize: Whether to tokenize the input before passing it to the vLLM engine.
-            If not, vLLM will tokenize the prompt in the engine.
-        detokenize: Whether to detokenize the output.
-        has_image: Whether the input messages have images.
+        should_continue_on_error: If True, continue processing when inference fails for a row
+            instead of raising an exception. Failed rows will have a non-null
+            ``__inference_error__`` column containing the error message, and other
+            output columns will be None. Error rows bypass postprocess. If False
+            (default), any inference error will raise an exception.
+        chat_template_stage: Chat templating stage config (bool | dict | ChatTemplateStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, and memory. Legacy ``apply_chat_template``
+            and ``chat_template`` fields are deprecated but still supported.
+        tokenize_stage: Tokenizer stage config (bool | dict | TokenizerStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, memory, and model_source. Legacy
+            ``tokenize`` field is deprecated but still supported.
+        detokenize_stage: Detokenizer stage config (bool | dict | DetokenizeStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, memory, and model_source. Legacy
+            ``detokenize`` field is deprecated but still supported.
+        prepare_image_stage: Prepare image stage config (bool | dict | PrepareImageStageConfig).
+            Defaults to False. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, and memory. Both the legacy ``has_image`` field
+            and ``prepare_image_stage`` are deprecated but still supported. Prefer to use multimodal
+            processor to process multimodal data instead.
         accelerator_type: The accelerator type used by the LLM stage in a processor.
             Default to None, meaning that only the CPU will be used.
         concurrency: The number of workers for data parallelism. Default to 1.
@@ -127,6 +153,7 @@ class vLLMEngineProcessorConfig(_vLLMEngineProcessorConfig):
             actor pool that scales between ``m`` and ``n`` workers (``1 <= m <= n``).
             If ``concurrency`` is an ``int`` ``n``, CPU stages use an autoscaling
             pool from ``(1, n)``, while GPU stages use a fixed pool of ``n`` workers.
+            Stage-specific concurrency can be set via nested stage configs.
 
     Examples:
 
@@ -134,7 +161,7 @@ class vLLMEngineProcessorConfig(_vLLMEngineProcessorConfig):
             :skipif: True
 
             import ray
-            from ray.data.llm import vLLMEngineProcessorConfig, build_llm_processor
+            from ray.data.llm import vLLMEngineProcessorConfig, build_processor
 
             config = vLLMEngineProcessorConfig(
                 model_source="meta-llama/Meta-Llama-3.1-8B-Instruct",
@@ -146,7 +173,7 @@ class vLLMEngineProcessorConfig(_vLLMEngineProcessorConfig):
                 concurrency=1,
                 batch_size=64,
             )
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     messages=[
@@ -205,12 +232,18 @@ class SGLangEngineProcessorConfig(_SGLangEngineProcessorConfig):
             each batch. The default value may not be optimal when the batch size
             or the batch processing latency is too small, but it should be good
             enough for batch size >= 64.
-        apply_chat_template: Whether to apply chat template.
-        chat_template: The chat template to use. This is usually not needed if the
-            model checkpoint already contains the chat template.
-        tokenize: Whether to tokenize the input before passing it to the SGLang engine.
-            If not, SGLang will tokenize the prompt in the engine.
-        detokenize: Whether to detokenize the output.
+        chat_template_stage: Chat templating stage config (bool | dict | ChatTemplateStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, and memory. Legacy ``apply_chat_template``
+            and ``chat_template`` fields are deprecated but still supported.
+        tokenize_stage: Tokenizer stage config (bool | dict | TokenizerStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, memory, and model_source. Legacy
+            ``tokenize`` field is deprecated but still supported.
+        detokenize_stage: Detokenizer stage config (bool | dict | DetokenizeStageConfig).
+            Defaults to True. Use nested config for per-stage control over batch_size,
+            concurrency, runtime_env, num_cpus, memory, and model_source. Legacy
+            ``detokenize`` field is deprecated but still supported.
         accelerator_type: The accelerator type used by the LLM stage in a processor.
             Default to None, meaning that only the CPU will be used.
         concurrency: The number of workers for data parallelism. Default to 1.
@@ -218,13 +251,14 @@ class SGLangEngineProcessorConfig(_SGLangEngineProcessorConfig):
             actor pool that scales between ``m`` and ``n`` workers (``1 <= m <= n``).
             If ``concurrency`` is an ``int`` ``n``, CPU stages use an autoscaling
             pool from ``(1, n)``, while GPU stages use a fixed pool of ``n`` workers.
+            Stage-specific concurrency can be set via nested stage configs.
 
     Examples:
         .. testcode::
             :skipif: True
 
             import ray
-            from ray.data.llm import SGLangEngineProcessorConfig, build_llm_processor
+            from ray.data.llm import SGLangEngineProcessorConfig, build_processor
 
             config = SGLangEngineProcessorConfig(
                 model_source="meta-llama/Meta-Llama-3.1-8B-Instruct",
@@ -234,7 +268,7 @@ class SGLangEngineProcessorConfig(_SGLangEngineProcessorConfig):
                 concurrency=1,
                 batch_size=64,
             )
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     messages=[
@@ -287,7 +321,7 @@ class ServeDeploymentProcessorConfig(_ServeDeploymentProcessorConfig):
 
             import ray
             from ray import serve
-            from ray.data.llm import ServeDeploymentProcessorConfig, build_llm_processor
+            from ray.data.llm import ServeDeploymentProcessorConfig, build_processor
             from ray.serve.llm import (
                 LLMConfig,
                 ModelLoadingConfig,
@@ -333,7 +367,7 @@ class ServeDeploymentProcessorConfig(_ServeDeploymentProcessorConfig):
                 concurrency=1,
                 batch_size=64,
             )
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     method="completions",
@@ -364,6 +398,141 @@ class ServeDeploymentProcessorConfig(_ServeDeploymentProcessorConfig):
 
 
 @PublicAPI(stability="alpha")
+class ChatTemplateStageConfig(_ChatTemplateStageConfig):
+    """The configuration for the chat template stage.
+
+    Args:
+        enabled: Whether this stage is enabled. Defaults to True.
+        model_source: Model source/identifier for this stage. If not specified,
+            will use the processor-level model_source.
+        chat_template: The chat template in Jinja template format. This is
+            usually not needed if the model checkpoint already contains the
+            chat template.
+        chat_template_kwargs: Optional kwargs to pass to apply_chat_template.
+        batch_size: Rows per batch. If not specified, will use the processor-level
+            batch_size.
+        concurrency: Actor pool size or range for this stage. If not specified,
+            will use the processor-level concurrency. If ``concurrency`` is a
+            tuple ``(m, n)``, Ray creates an autoscaling actor pool that scales
+            between ``m`` and ``n`` workers (``1 <= m <= n``). If ``concurrency``
+            is an ``int`` ``n``, CPU stages use an autoscaling pool from ``(1, n)``.
+        runtime_env: Optional runtime environment for this stage. If not specified,
+            will use the processor-level runtime_env. See
+            :ref:`this doc <handling_dependencies>` for more details.
+        num_cpus: Number of CPUs to reserve for each map worker in this stage.
+        memory: Heap memory in bytes to reserve for each map worker in this stage.
+    """
+
+    pass
+
+
+@PublicAPI(stability="alpha")
+class DetokenizeStageConfig(_DetokenizeStageConfig):
+    """The configuration for the detokenize stage.
+
+    Args:
+        enabled: Whether this stage is enabled. Defaults to True.
+        model_source: Model source/identifier for this stage. If not specified,
+            will use the processor-level model_source.
+        batch_size: Rows per batch. If not specified, will use the processor-level
+            batch_size.
+        concurrency: Actor pool size or range for this stage. If not specified,
+            will use the processor-level concurrency. If ``concurrency`` is a
+            tuple ``(m, n)``, Ray creates an autoscaling actor pool that scales
+            between ``m`` and ``n`` workers (``1 <= m <= n``). If ``concurrency``
+            is an ``int`` ``n``, CPU stages use an autoscaling pool from ``(1, n)``.
+        runtime_env: Optional runtime environment for this stage. If not specified,
+            will use the processor-level runtime_env. See
+            :ref:`this doc <handling_dependencies>` for more details.
+        num_cpus: Number of CPUs to reserve for each map worker in this stage.
+        memory: Heap memory in bytes to reserve for each map worker in this stage.
+    """
+
+    pass
+
+
+@PublicAPI(stability="alpha")
+class PrepareMultimodalStageConfig(_PrepareMultimodalStageConfig):
+    """The configuration for the prepare multimodal stage.
+
+    Args:
+        enabled: Whether this stage is enabled. Defaults to True.
+        model_source: Name or path of the Hugging Face model to use for the
+            multimodal processor. This is required to process multimodal data
+            according to a specific model. If not specified, will use the
+            processor-level model_source.
+        chat_template_content_format: The content format to use for the chat
+            template. This is used to format the chat template content according
+            to a specific model. Choices are "string" or "openai". Defaults to
+            "string".
+        apply_sys_msg_formatting: Whether to apply formatting system messages.
+            Defaults to False.
+        batch_size: Rows per batch. If not specified, will use the processor-level
+            batch_size.
+        concurrency: Actor pool size or range for this stage. If not specified,
+            will use the processor-level concurrency. If ``concurrency`` is a
+            tuple ``(m, n)``, Ray creates an autoscaling actor pool that scales
+            between ``m`` and ``n`` workers (``1 <= m <= n``). If ``concurrency``
+            is an ``int`` ``n``, CPU stages use an autoscaling pool from ``(1, n)``.
+        runtime_env: Optional runtime environment for this stage. If not specified,
+            will use the processor-level runtime_env. See
+            :ref:`this doc <handling_dependencies>` for more details.
+        num_cpus: Number of CPUs to reserve for each map worker in this stage.
+        memory: Heap memory in bytes to reserve for each map worker in this stage.
+    """
+
+    pass
+
+
+@PublicAPI(stability="alpha")
+class TokenizerStageConfig(_TokenizerStageConfig):
+    """The configuration for the tokenizer stage.
+
+    Args:
+        enabled: Whether this stage is enabled. Defaults to True.
+        model_source: Model source/identifier for this stage. If not specified,
+            will use the processor-level model_source.
+        batch_size: Rows per batch. If not specified, will use the processor-level
+            batch_size.
+        concurrency: Actor pool size or range for this stage. If not specified,
+            will use the processor-level concurrency. If ``concurrency`` is a
+            tuple ``(m, n)``, Ray creates an autoscaling actor pool that scales
+            between ``m`` and ``n`` workers (``1 <= m <= n``). If ``concurrency``
+            is an ``int`` ``n``, CPU stages use an autoscaling pool from ``(1, n)``.
+        runtime_env: Optional runtime environment for this stage. If not specified,
+            will use the processor-level runtime_env. See
+            :ref:`this doc <handling_dependencies>` for more details.
+        num_cpus: Number of CPUs to reserve for each map worker in this stage.
+        memory: Heap memory in bytes to reserve for each map worker in this stage.
+    """
+
+    pass
+
+
+@PublicAPI(stability="alpha")
+class PrepareImageStageConfig(_PrepareImageStageConfig):
+    """The configuration for the prepare image stage.
+
+    Args:
+        enabled: Whether this stage is enabled. Defaults to True.
+        batch_size: Rows per batch. If not specified, will use the processor-level
+            batch_size.
+        concurrency: Actor pool size or range for this stage. If not specified,
+            will use the processor-level concurrency. If ``concurrency`` is a
+            tuple ``(m, n)``, Ray creates an autoscaling actor pool that scales
+            between ``m`` and ``n`` workers (``1 <= m <= n``). If ``concurrency``
+            is an ``int`` ``n``, CPU stages use an autoscaling pool from ``(1, n)``.
+        runtime_env: Optional runtime environment for this stage. If not specified,
+            will use the processor-level runtime_env. See
+            :ref:`this doc <handling_dependencies>` for more details.
+        num_cpus: Number of CPUs to reserve for each map worker in this stage.
+        memory: Heap memory in bytes to reserve for each map worker in this stage.
+    """
+
+    pass
+
+
+@Deprecated(new="build_processor", error=False)
 def build_llm_processor(
     config: ProcessorConfig,
     preprocess: Optional[UserDefinedFunction] = None,
@@ -372,10 +541,37 @@ def build_llm_processor(
     postprocess_map_kwargs: Optional[Dict[str, Any]] = None,
     builder_kwargs: Optional[Dict[str, Any]] = None,
 ) -> Processor:
-    """Build a LLM processor using the given config.
+    """
+    [DEPRECATED] Prefer build_processor. Build a LLM processor using the given config.
+    """
+    return build_processor(
+        config,
+        preprocess,
+        postprocess,
+        preprocess_map_kwargs,
+        postprocess_map_kwargs,
+        builder_kwargs,
+    )
+
+
+@PublicAPI(stability="alpha")
+def build_processor(
+    config: ProcessorConfig,
+    preprocess: Optional[UserDefinedFunction] = None,
+    postprocess: Optional[UserDefinedFunction] = None,
+    preprocess_map_kwargs: Optional[Dict[str, Any]] = None,
+    postprocess_map_kwargs: Optional[Dict[str, Any]] = None,
+    builder_kwargs: Optional[Dict[str, Any]] = None,
+) -> Processor:
+    """Build a processor using the given config.
 
     Args:
-        config: The processor config.
+        config: The processor config. Supports nested stage configs for per-stage
+            control over batch_size, concurrency, runtime_env, num_cpus, and memory
+            (e.g., ``chat_template_stage=ChatTemplateStageConfig(batch_size=128)``
+            or ``tokenize_stage={"batch_size": 256, "concurrency": 2}``). Legacy
+            boolean flags (``apply_chat_template``, ``tokenize``, ``detokenize``,
+            ``has_image``) are deprecated but still supported with deprecation warnings.
         preprocess: An optional lambda function that takes a row (dict) as input
             and returns a preprocessed row (dict). The output row must contain the
             required fields for the following processing stages. Each row
@@ -406,7 +602,7 @@ def build_llm_processor(
             :skipif: True
 
             import ray
-            from ray.data.llm import vLLMEngineProcessorConfig, build_llm_processor
+            from ray.data.llm import vLLMEngineProcessorConfig, build_processor
 
             config = vLLMEngineProcessorConfig(
                 model_source="meta-llama/Meta-Llama-3.1-8B-Instruct",
@@ -419,7 +615,7 @@ def build_llm_processor(
                 batch_size=64,
             )
 
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     messages=[
@@ -449,7 +645,7 @@ def build_llm_processor(
             :skipif: True
 
             import ray
-            from ray.data.llm import vLLMEngineProcessorConfig, build_llm_processor
+            from ray.data.llm import vLLMEngineProcessorConfig, build_processor
 
             config = vLLMEngineProcessorConfig(
                 model_source="meta-llama/Meta-Llama-3.1-8B-Instruct",
@@ -457,7 +653,7 @@ def build_llm_processor(
                 batch_size=64,
             )
 
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     messages=[{"role": "user", "content": row["prompt"]}],
@@ -479,16 +675,16 @@ def build_llm_processor(
             :skipif: True
 
             import ray
-            from ray.data.llm import vLLMEngineProcessorConfig, build_llm_processor
+            from ray.data.llm import vLLMEngineProcessorConfig, build_processor
 
             config = vLLMEngineProcessorConfig(
                 model_source="Qwen/Qwen3-0.6B",
-                apply_chat_template=True,
+                chat_template_stage={"enabled": True},
                 concurrency=1,
                 batch_size=64,
             )
 
-            processor = build_llm_processor(
+            processor = build_processor(
                 config,
                 preprocess=lambda row: dict(
                     messages=[
@@ -534,5 +730,11 @@ __all__ = [
     "vLLMEngineProcessorConfig",
     "SGLangEngineProcessorConfig",
     "ServeDeploymentProcessorConfig",
+    "ChatTemplateStageConfig",
+    "DetokenizeStageConfig",
+    "PrepareMultimodalStageConfig",
+    "TokenizerStageConfig",
+    "PrepareImageStageConfig",
     "build_llm_processor",
+    "build_processor",
 ]
