@@ -1,16 +1,20 @@
-from typing import List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple
 
 import numpy as np
 import pandas as pd
 
-from ray.data import Dataset
-from ray.data.aggregate import AbsMax, Max, Mean, Min, Std
-from ray.data.preprocessor import Preprocessor
+from ray.data.aggregate import AbsMax, ApproximateQuantile, Max, Mean, Min, Std
+from ray.data.preprocessor import Preprocessor, SerializablePreprocessorBase
+from ray.data.preprocessors.version_support import SerializablePreprocessor
 from ray.util.annotations import PublicAPI
 
+if TYPE_CHECKING:
+    from ray.data.dataset import Dataset
+
 
 @PublicAPI(stability="alpha")
-class StandardScaler(Preprocessor):
+@SerializablePreprocessor(version=1, identifier="io.ray.preprocessors.standard_scaler")
+class StandardScaler(SerializablePreprocessorBase):
     r"""Translate and scale each column by its mean and standard deviation,
     respectively.
 
@@ -79,15 +83,21 @@ class StandardScaler(Preprocessor):
     """
 
     def __init__(self, columns: List[str], output_columns: Optional[List[str]] = None):
+        super().__init__()
         self.columns = columns
         self.output_columns = Preprocessor._derive_and_validate_output_columns(
             columns, output_columns
         )
 
-    def _fit(self, dataset: Dataset) -> Preprocessor:
-        mean_aggregates = [Mean(col) for col in self.columns]
-        std_aggregates = [Std(col, ddof=0) for col in self.columns]
-        self.stats_ = dataset.aggregate(*mean_aggregates, *std_aggregates)
+    def _fit(self, dataset: "Dataset") -> Preprocessor:
+        self.stat_computation_plan.add_aggregator(
+            aggregator_fn=Mean,
+            columns=self.columns,
+        )
+        self.stat_computation_plan.add_aggregator(
+            aggregator_fn=lambda col: Std(col, ddof=0),
+            columns=self.columns,
+        )
         return self
 
     def _transform_pandas(self, df: pd.DataFrame):
@@ -109,12 +119,27 @@ class StandardScaler(Preprocessor):
         df[self.output_columns] = df[self.columns].transform(column_standard_scaler)
         return df
 
+    def _get_serializable_fields(self) -> Dict[str, Any]:
+        return {
+            "columns": self.columns,
+            "output_columns": self.output_columns,
+            "_fitted": getattr(self, "_fitted", None),
+        }
+
+    def _set_serializable_fields(self, fields: Dict[str, Any], version: int):
+        # required fields
+        self.columns = fields["columns"]
+        self.output_columns = fields["output_columns"]
+        # optional fields
+        self._fitted = fields.get("_fitted")
+
     def __repr__(self):
         return f"{self.__class__.__name__}(columns={self.columns!r}, output_columns={self.output_columns!r})"
 
 
 @PublicAPI(stability="alpha")
-class MinMaxScaler(Preprocessor):
+@SerializablePreprocessor(version=1, identifier="io.ray.preprocessors.min_max_scaler")
+class MinMaxScaler(SerializablePreprocessorBase):
     r"""Scale each column by its range.
 
     The general formula is given by
@@ -179,12 +204,13 @@ class MinMaxScaler(Preprocessor):
     """
 
     def __init__(self, columns: List[str], output_columns: Optional[List[str]] = None):
+        super().__init__()
         self.columns = columns
         self.output_columns = Preprocessor._derive_and_validate_output_columns(
             columns, output_columns
         )
 
-    def _fit(self, dataset: Dataset) -> Preprocessor:
+    def _fit(self, dataset: "Dataset") -> Preprocessor:
         aggregates = [Agg(col) for Agg in [Min, Max] for col in self.columns]
         self.stats_ = dataset.aggregate(*aggregates)
         return self
@@ -205,12 +231,27 @@ class MinMaxScaler(Preprocessor):
         df[self.output_columns] = df[self.columns].transform(column_min_max_scaler)
         return df
 
+    def _get_serializable_fields(self) -> Dict[str, Any]:
+        return {
+            "columns": self.columns,
+            "output_columns": self.output_columns,
+            "_fitted": getattr(self, "_fitted", None),
+        }
+
+    def _set_serializable_fields(self, fields: Dict[str, Any], version: int):
+        # required fields
+        self.columns = fields["columns"]
+        self.output_columns = fields["output_columns"]
+        # optional fields
+        self._fitted = fields.get("_fitted")
+
     def __repr__(self):
         return f"{self.__class__.__name__}(columns={self.columns!r}, output_columns={self.output_columns!r})"
 
 
 @PublicAPI(stability="alpha")
-class MaxAbsScaler(Preprocessor):
+@SerializablePreprocessor(version=1, identifier="io.ray.preprocessors.max_abs_scaler")
+class MaxAbsScaler(SerializablePreprocessorBase):
     r"""Scale each column by its absolute max value.
 
     The general formula is given by
@@ -271,12 +312,13 @@ class MaxAbsScaler(Preprocessor):
     """
 
     def __init__(self, columns: List[str], output_columns: Optional[List[str]] = None):
+        super().__init__()
         self.columns = columns
         self.output_columns = Preprocessor._derive_and_validate_output_columns(
             columns, output_columns
         )
 
-    def _fit(self, dataset: Dataset) -> Preprocessor:
+    def _fit(self, dataset: "Dataset") -> Preprocessor:
         aggregates = [AbsMax(col) for col in self.columns]
         self.stats_ = dataset.aggregate(*aggregates)
         return self
@@ -295,13 +337,28 @@ class MaxAbsScaler(Preprocessor):
         df[self.output_columns] = df[self.columns].transform(column_abs_max_scaler)
         return df
 
+    def _get_serializable_fields(self) -> Dict[str, Any]:
+        return {
+            "columns": self.columns,
+            "output_columns": self.output_columns,
+            "_fitted": getattr(self, "_fitted", None),
+        }
+
+    def _set_serializable_fields(self, fields: Dict[str, Any], version: int):
+        # required fields
+        self.columns = fields["columns"]
+        self.output_columns = fields["output_columns"]
+        # optional fields
+        self._fitted = fields.get("_fitted")
+
     def __repr__(self):
         return f"{self.__class__.__name__}(columns={self.columns!r}, output_columns={self.output_columns!r})"
 
 
 @PublicAPI(stability="alpha")
-class RobustScaler(Preprocessor):
-    r"""Scale and translate each column using quantiles.
+@SerializablePreprocessor(version=1, identifier="io.ray.preprocessors.robust_scaler")
+class RobustScaler(SerializablePreprocessorBase):
+    r"""Scale and translate each column using approximate quantiles.
 
     The general formula is given by
 
@@ -313,6 +370,9 @@ class RobustScaler(Preprocessor):
     high and low quantiles, respectively. By default, :math:`\mu_{h}` is the third
     quartile and :math:`\mu_{l}` is the first quartile.
 
+    Internally, the `ApproximateQuantile` aggregator is used to calculate the
+    approximate quantiles.
+
     .. tip::
         This scaler works well when your data contains many outliers.
 
@@ -367,52 +427,52 @@ class RobustScaler(Preprocessor):
             columns will be the same as the input columns. If not None, the length of
             ``output_columns`` must match the length of ``columns``, othwerwise an error
             will be raised.
+        quantile_precision: Controls the accuracy and memory footprint of the sketch (K in KLL);
+            higher values yield lower error but use more memory. Defaults to 800. See
+            https://datasketches.apache.org/docs/KLL/KLLAccuracyAndSize.html
+            for details on accuracy and size.
     """
 
+    DEFAULT_QUANTILE_PRECISION = 800
+
     def __init__(
         self,
         columns: List[str],
         quantile_range: Tuple[float, float] = (0.25, 0.75),
         output_columns: Optional[List[str]] = None,
+        quantile_precision: int = DEFAULT_QUANTILE_PRECISION,
     ):
+        super().__init__()
         self.columns = columns
         self.quantile_range = quantile_range
+        self.quantile_precision = quantile_precision
 
         self.output_columns = Preprocessor._derive_and_validate_output_columns(
             columns, output_columns
         )
 
-    def _fit(self, dataset: Dataset) -> Preprocessor:
-        low = self.quantile_range[0]
-        med = 0.50
-        high = self.quantile_range[1]
-
-        num_records = dataset.count()
-        max_index = num_records - 1
-        split_indices = [int(percentile * max_index) for percentile in (low, med, high)]
+    def _fit(self, dataset: "Dataset") -> Preprocessor:
+        quantiles = [
+            self.quantile_range[0],
+            0.50,
+            self.quantile_range[1],
+        ]
+        aggregates = [
+            ApproximateQuantile(
+                on=col,
+                quantiles=quantiles,
+                quantile_precision=self.quantile_precision,
+            )
+            for col in self.columns
+        ]
+        aggregated = dataset.aggregate(*aggregates)
 
         self.stats_ = {}
-
-        # TODO(matt): Handle case where quantile lands between 2 numbers.
-        # The current implementation will simply choose the closest index.
-        # This will affect the results of small datasets more than large datasets.
         for col in self.columns:
-            filtered_dataset = dataset.map_batches(
-                lambda df: df[[col]], batch_format="pandas"
-            )
-            sorted_dataset = filtered_dataset.sort(col)
-            _, low, med, high = sorted_dataset.split_at_indices(split_indices)
-
-            def _get_first_value(ds: Dataset, c: str):
-                return ds.take(1)[0][c]
-
-            low_val = _get_first_value(low, col)
-            med_val = _get_first_value(med, col)
-            high_val = _get_first_value(high, col)
-
-            self.stats_[f"low_quantile({col})"] = low_val
-            self.stats_[f"median({col})"] = med_val
-            self.stats_[f"high_quantile({col})"] = high_val
+            low_q, med_q, high_q = aggregated[f"approx_quantile({col})"]
+            self.stats_[f"low_quantile({col})"] = low_q
+            self.stats_[f"median({col})"] = med_q
+            self.stats_[f"high_quantile({col})"] = high_q
 
         return self
 
@@ -433,6 +493,24 @@ class RobustScaler(Preprocessor):
         df[self.output_columns] = df[self.columns].transform(column_robust_scaler)
         return df
 
+    def _get_serializable_fields(self) -> Dict[str, Any]:
+        return {
+            "columns": self.columns,
+            "output_columns": self.output_columns,
+            "quantile_range": self.quantile_range,
+            "quantile_precision": self.quantile_precision,
+            "_fitted": getattr(self, "_fitted", None),
+        }
+
+    def _set_serializable_fields(self, fields: Dict[str, Any], version: int):
+        # required fields
+        self.columns = fields["columns"]
+        self.output_columns = fields["output_columns"]
+        self.quantile_range = fields["quantile_range"]
+        self.quantile_precision = fields["quantile_precision"]
+        # optional fields
+        self._fitted = fields.get("_fitted")
+
     def __repr__(self):
         return (
             f"{self.__class__.__name__}(columns={self.columns!r}, "
