@@ -4,8 +4,18 @@ from ray.llm._internal.batch.stages.base import (
     wrap_preprocess,
 )
 from ray.llm._internal.batch.stages.chat_template_stage import ChatTemplateStage
+from ray.llm._internal.batch.stages.configs import (
+    ChatTemplateStageConfig,
+    DetokenizeStageConfig,
+    PrepareImageStageConfig,
+    PrepareMultimodalStageConfig,
+    TokenizerStageConfig,
+)
 from ray.llm._internal.batch.stages.http_request_stage import HttpRequestStage
 from ray.llm._internal.batch.stages.prepare_image_stage import PrepareImageStage
+from ray.llm._internal.batch.stages.prepare_multimodal_stage import (
+    PrepareMultimodalStage,
+)
 from ray.llm._internal.batch.stages.serve_deployment_stage import ServeDeploymentStage
 from ray.llm._internal.batch.stages.sglang_engine_stage import SGLangEngineStage
 from ray.llm._internal.batch.stages.tokenize_stage import DetokenizeStage, TokenizeStage
@@ -14,6 +24,7 @@ from ray.llm._internal.batch.stages.vllm_engine_stage import vLLMEngineStage
 __all__ = [
     "StatefulStage",
     "HttpRequestStage",
+    "PrepareMultimodalStage",
     "ChatTemplateStage",
     "TokenizeStage",
     "DetokenizeStage",
@@ -23,4 +34,9 @@ __all__ = [
     "wrap_preprocess",
     "wrap_postprocess",
     "PrepareImageStage",
+    "ChatTemplateStageConfig",
+    "DetokenizeStageConfig",
+    "PrepareImageStageConfig",
+    "PrepareMultimodalStageConfig",
+    "TokenizerStageConfig",
 ]
