@@ -10,12 +10,17 @@ import uuid
 from collections import Counter
 from enum import Enum
 from functools import partial
-from typing import Any, AsyncIterator, Dict, List, Optional, Tuple, Type
+from typing import TYPE_CHECKING, Any, AsyncIterator, Dict, List, Optional, Tuple, Type
 
 import numpy as np
 import torch
 from pydantic import BaseModel, Field, root_validator
 
+if TYPE_CHECKING:
+    from vllm.multimodal import MultiModalDataDict
+else:
+    MultiModalDataDict = Any
+
 import ray
 from ray.llm._internal.batch.stages.base import (
     StatefulStage,
@@ -33,6 +38,21 @@ from ray.util.scheduling_strategies import PlacementGroupSchedulingStrategy
 
 logger = logging.getLogger(__name__)
 
+# vLLM fatal errors that should always be re-raised, never swallowed.
+# EngineDeadError indicates the vLLM engine process has crashed and is
+# unrecoverable - all subsequent requests would fail anyway.
+_VLLM_FATAL_ERRORS: Tuple[Type[Exception], ...] = ()
+try:
+    from vllm.v1.engine.exceptions import EngineDeadError
+
+    _VLLM_FATAL_ERRORS = (EngineDeadError,)
+except ImportError:
+    # vLLM not installed or older version without this exception
+    pass
+
+# Length of prompt snippet to surface in case of recoverable error
+_MAX_PROMPT_LENGTH_IN_ERROR = 500
+
 
 class vLLMTaskType(str, Enum):
     """The type of task to run on the vLLM engine."""
@@ -53,8 +73,14 @@ class vLLMEngineRequest(BaseModel):
     idx_in_batch: int
     # The full prompt string (with chat template applied if any).
     prompt: str
-    # The images inputs for the multimodal model. Use Any to avoid importing PIL.
+    # DEPRECATED: The images inputs for the multimodal model. Use Any to avoid importing PIL.
     images: List[Any]
+    # The multimodal data for the multimodal model.
+    multimodal_data: Optional[MultiModalDataDict]
+    # The kwargs for the multimodal processor.
+    mm_processor_kwargs: Optional[Dict[str, Any]]
+    # The uuids for the multimodal data.
+    multimodal_uuids: Optional[Dict[str, Any]]
     # The tokenized prompt IDs. If None, then the string prompt will be
     # tokenized by the LLM engine. This is not recommended for performance reasons.
     prompt_token_ids: Optional[List[int]]
@@ -68,6 +94,38 @@ class vLLMEngineRequest(BaseModel):
         arbitrary_types_allowed = True
 
 
+def _convert_logprob_dict(logprob_dict: Dict[int, Any]) -> Dict[int, Dict[str, Any]]:
+    """Convert a dict of token_id -> Logprob to token_id -> dict.
+
+    Handles conversion of vLLM's Logprob objects (currently dataclass) to
+    serializable dicts. This supports both dataclass (current vLLM format)
+    and Pydantic models (for future compatibility).
+
+    Args:
+        logprob_dict: Dict mapping token_id to Logprob instance.
+
+    Returns:
+        Dict mapping token_id to serializable dict with logprob fields.
+    """
+    result = {}
+    for token_id, logprob in logprob_dict.items():
+        # Handle Pydantic models (model_dump method)
+        if hasattr(logprob, "model_dump"):
+            result[token_id] = logprob.model_dump()
+        # Handle dataclasses (current vLLM format)
+        elif dataclasses.is_dataclass(logprob):
+            result[token_id] = dataclasses.asdict(logprob)
+        # Already a dict
+        elif isinstance(logprob, dict):
+            result[token_id] = logprob
+        else:
+            raise TypeError(
+                f"Unsupported logprob type: {type(logprob)}. "
+                "Expected dataclass, Pydantic model, or dict."
+            )
+    return result
+
+
 class vLLMOutputData(BaseModel):
     """The output of the vLLM engine."""
 
@@ -87,6 +145,14 @@ class vLLMOutputData(BaseModel):
     # Metrics fields.
     metrics: Optional[Dict[str, Any]] = None
 
+    # Logprobs fields.
+    # logprobs: List[Dict[int, Dict[str, Any]]] where each dict maps token_id to
+    # logprob info (logprob, rank, decoded_token) for each generated token.
+    logprobs: Optional[List[Dict[int, Dict[str, Any]]]] = None
+    # prompt_logprobs: List[Optional[Dict[int, Dict[str, Any]]]] where each dict
+    # (or None) maps token_id to logprob info for each prompt token.
+    prompt_logprobs: Optional[List[Optional[Dict[int, Dict[str, Any]]]]] = None
+
     @classmethod
     def from_vllm_engine_output(cls, output: Any) -> "vLLMOutputData":
         """Create a vLLMOutputData from a vLLM engine output."""
@@ -111,6 +177,22 @@ class vLLMOutputData(BaseModel):
             data.generated_tokens = output.outputs[0].token_ids
             data.generated_text = output.outputs[0].text
             data.num_generated_tokens = len(output.outputs[0].token_ids)
+
+            # Extract logprobs
+            if output.outputs[0].logprobs is not None:
+                data.logprobs = [
+                    _convert_logprob_dict(logprob_dict)
+                    for logprob_dict in output.outputs[0].logprobs
+                ]
+
+            # Extract prompt_logprobs
+            if output.prompt_logprobs is not None:
+                data.prompt_logprobs = [
+                    _convert_logprob_dict(logprob_dict)
+                    if logprob_dict is not None
+                    else None
+                    for logprob_dict in output.prompt_logprobs
+                ]
         elif isinstance(output, vllm.outputs.PoolingRequestOutput):
             data.embeddings = output.outputs.data.cpu()
             if (
@@ -268,6 +350,12 @@ class vLLMEngineWrapper:
         else:
             image = []
 
+        multimodal_data = row.pop("multimodal_data", None)
+        # TODO (jeffreywang): As we decouple the multimodal processor from the vLLM engine,
+        # these kwargs are not needed in the vLLM engine stage.
+        mm_processor_kwargs = row.pop("mm_processor_kwargs", None)
+        multimodal_uuids = row.pop("multimodal_uuids", None)
+
         lora_request = await self._maybe_get_lora_request(row)
 
         # Prepare sampling parameters.
@@ -276,16 +364,16 @@ class vLLMEngineWrapper:
         if self.task_type == vLLMTaskType.GENERATE:
             sampling_params = row.pop("sampling_params")
             if "guided_decoding" in sampling_params:
-                guided_decoding = vllm.sampling_params.GuidedDecodingParams(
+                structured_outputs = vllm.sampling_params.StructuredOutputsParams(
                     **maybe_convert_ndarray_to_list(
                         sampling_params.pop("guided_decoding")
                     )
                 )
             else:
-                guided_decoding = None
+                structured_outputs = None
             params = vllm.SamplingParams(
                 **maybe_convert_ndarray_to_list(sampling_params),
-                guided_decoding=guided_decoding,
+                structured_outputs=structured_outputs,
             )
         elif self.task_type == vLLMTaskType.EMBED:
             params = vllm.PoolingParams(task=self.task_type.value)
@@ -298,6 +386,9 @@ class vLLMEngineWrapper:
             prompt=prompt,
             prompt_token_ids=tokenized_prompt,
             images=image,
+            multimodal_data=multimodal_data,
+            mm_processor_kwargs=mm_processor_kwargs,
+            multimodal_uuids=multimodal_uuids,
             params=params,
             lora_request=lora_request,
         )
@@ -336,25 +427,48 @@ class vLLMEngineWrapper:
             The output of the request.
         """
 
-        # NOTE: vLLM v1 tighly couples tokenizer and detokenizer to the engine.
-        # We should investigate whether decoupling them could lead to better
-        # performance. Given that v1 tokenizer and detokenizer are already
-        # in a separate process, the benefit of decoupling them in the Processor
-        # may be limited.
-        assert request.prompt
         import vllm
 
-        multi_modal_data = {"image": request.images} if request.images else None
-        llm_prompt = vllm.inputs.data.TextPrompt(
-            prompt=request.prompt, multi_modal_data=multi_modal_data
-        )
+        # TODO (jeffreywang): Consolidate to multimodal_data only
+        if request.images:
+            multi_modal_data = (
+                {**request.multimodal_data, "image": request.images}
+                if request.multimodal_data
+                else {"image": request.images}
+            )
+        else:
+            multi_modal_data = request.multimodal_data
+
+        if request.prompt_token_ids is not None:
+            llm_prompt = vllm.inputs.data.TokensPrompt(
+                prompt_token_ids=request.prompt_token_ids,
+                multi_modal_data=multi_modal_data,
+                mm_processor_kwargs=request.mm_processor_kwargs,
+                multi_modal_uuids=request.multimodal_uuids,
+            )
+        else:
+            assert request.prompt
+            llm_prompt = vllm.inputs.data.TextPrompt(
+                prompt=request.prompt,
+                multi_modal_data=multi_modal_data,
+                mm_processor_kwargs=request.mm_processor_kwargs,
+                multi_modal_uuids=request.multimodal_uuids,
+            )
 
         # Send the request to the LLM engine.
-        stream = self.engine.generate(
-            request_id=str(request.request_id),
-            prompt=llm_prompt,
-            sampling_params=request.params,
-        )
+        # vLLM 0.12.0 uses encode() for pooling/embedding tasks, generate() for text generation
+        if self.task_type == vLLMTaskType.EMBED:
+            stream = self.engine.encode(
+                request_id=str(request.request_id),
+                prompt=llm_prompt,
+                pooling_params=request.params,
+            )
+        else:
+            stream = self.engine.generate(
+                request_id=str(request.request_id),
+                prompt=llm_prompt,
+                sampling_params=request.params,
+            )
 
         # Consume the stream until the request is finished.
         async for request_output in stream:
@@ -393,6 +507,7 @@ class vLLMEngineStageUDF(StatefulStageUDF):
         task_type: vLLMTaskType = vLLMTaskType.GENERATE,
         max_pending_requests: Optional[int] = None,
         dynamic_lora_loading_path: Optional[str] = None,
+        should_continue_on_error: bool = False,
     ):
         """
         Initialize the vLLMEngineStageUDF.
@@ -407,9 +522,13 @@ class vLLMEngineStageUDF(StatefulStageUDF):
                 it will be set to 1.1 * max_num_seqs * pipeline_parallel_size.
             dynamic_lora_loading_path: The path to the dynamic LoRA adapter. It is expected
                 to hold subfolders each for a different lora checkpoint.
+            should_continue_on_error: If True, continue processing when inference fails for
+                a row instead of raising. Failed rows will have '__inference_error__'
+                set to the error message.
         """
         super().__init__(data_column, expected_input_keys)
         self.model = model
+        self.should_continue_on_error = should_continue_on_error
 
         # Setup vLLM engine kwargs.
         self.task_type = task_type
@@ -501,6 +620,57 @@ class vLLMEngineStageUDF(StatefulStageUDF):
         engine_kwargs["task"] = task_type
         return engine_kwargs
 
+    async def _generate_with_error_handling(
+        self,
+        row: Dict[str, Any],
+        batch_uuid: uuid.UUID,
+    ) -> Dict[str, Any]:
+        """Generate output for a single row, catching errors if should_continue_on_error is set.
+
+        Args:
+            row: The input row.
+            batch_uuid: The batch UUID for logging.
+
+        Returns:
+            The output dict, with __inference_error__ set if an error occurred.
+        """
+        idx_in_batch = row[self.IDX_IN_BATCH_COLUMN]
+        try:
+            request, output, time_taken_llm = await self.llm.generate_async(row)
+            return {
+                **output,
+                "request_id": request.request_id,
+                self.IDX_IN_BATCH_COLUMN: request.idx_in_batch,
+                "batch_uuid": batch_uuid.hex,
+                "time_taken_llm": time_taken_llm,
+                "params": str(request.params),
+                "__inference_error__": None,
+            }
+        except _VLLM_FATAL_ERRORS:
+            # Fatal engine errors (e.g., EngineDeadError) must always propagate.
+            # The engine is dead and all subsequent requests would fail.
+            raise
+        except Exception as e:
+            if not self.should_continue_on_error:
+                raise
+            error_msg = f"{type(e).__name__}: {str(e)}"
+            logger.warning(
+                "[vLLM] Inference failed for row %d in batch %s: %s",
+                idx_in_batch,
+                batch_uuid.hex,
+                error_msg,
+            )
+            # Include snippet of failed prompt
+            prompt = row.get("prompt", "")
+            if len(prompt) > _MAX_PROMPT_LENGTH_IN_ERROR:
+                prompt = prompt[:_MAX_PROMPT_LENGTH_IN_ERROR] + "...[truncated]"
+            return {
+                self.IDX_IN_BATCH_COLUMN: idx_in_batch,
+                "batch_uuid": batch_uuid.hex,
+                "__inference_error__": error_msg,
+                "prompt": prompt,
+            }
+
     async def udf(self, batch: List[Dict[str, Any]]) -> AsyncIterator[Dict[str, Any]]:
         """Run the vLLM engine.
 
@@ -513,19 +683,13 @@ class vLLMEngineStageUDF(StatefulStageUDF):
         batch_uuid = uuid.uuid4()
         batch_start_time = time.perf_counter()
 
-        tasks = [asyncio.create_task(self.llm.generate_async(row)) for row in batch]
+        tasks = [
+            asyncio.create_task(self._generate_with_error_handling(row, batch_uuid))
+            for row in batch
+        ]
 
         for resp in asyncio.as_completed(tasks):
-            request, output, time_taken_llm = await resp
-
-            yield {
-                **output,
-                "request_id": request.request_id,
-                self.IDX_IN_BATCH_COLUMN: request.idx_in_batch,
-                "batch_uuid": batch_uuid.hex,
-                "time_taken_llm": time_taken_llm,
-                "params": str(request.params),
-            }
+            yield await resp
 
         batch_time_taken = time.perf_counter() - batch_start_time
         # TODO: Add metrics to the UDf wrapper so that we don't need
@@ -695,4 +859,7 @@ class vLLMEngineStage(StatefulStage):
             "image": "The image(s) for multimodal input. Accepts a single image or list of images.",
             "model": "The model to use for this request. If the model is different from the "
             "model set in the stage, then this is a LoRA request.",
+            "multimodal_data": "The multimodal data to pass to the model, if the model supports it.",
+            "mm_processor_kwargs": "The kwargs for the engine's multimodal processor.",
+            "multimodal_uuids": "User-specified UUIDs for multimodal items, mapped by modality.",
         }
