@@ -6,11 +6,7 @@ they will be upstreamed to vLLM.
 
 from typing import TYPE_CHECKING, Any, AsyncGenerator, Dict, List, Optional, Union
 
-from pydantic import (
-    BaseModel,
-    ConfigDict,
-    Field,
-)
+from pydantic import BaseModel, ConfigDict, Field
 from vllm.entrypoints.openai.protocol import (
     ChatCompletionRequest as vLLMChatCompletionRequest,
     ChatCompletionResponse as vLLMChatCompletionResponse,
@@ -18,17 +14,21 @@ from vllm.entrypoints.openai.protocol import (
     CompletionRequest as vLLMCompletionRequest,
     CompletionResponse as vLLMCompletionResponse,
     CompletionStreamResponse as vLLMCompletionStreamResponse,
-    EmbeddingChatRequest as vLLMEmbeddingChatRequest,
-    EmbeddingCompletionRequest as vLLMEmbeddingCompletionRequest,
-    EmbeddingResponse as vLLMEmbeddingResponse,
     ErrorInfo as vLLMErrorInfo,
     ErrorResponse as vLLMErrorResponse,
-    ScoreRequest as vLLMScoreRequest,
-    ScoreResponse as vLLMScoreResponse,
     TranscriptionRequest as vLLMTranscriptionRequest,
     TranscriptionResponse as vLLMTranscriptionResponse,
     TranscriptionStreamResponse as vLLMTranscriptionStreamResponse,
 )
+from vllm.entrypoints.pooling.embed.protocol import (
+    EmbeddingChatRequest as vLLMEmbeddingChatRequest,
+    EmbeddingCompletionRequest as vLLMEmbeddingCompletionRequest,
+    EmbeddingResponse as vLLMEmbeddingResponse,
+)
+from vllm.entrypoints.pooling.score.protocol import (
+    ScoreRequest as vLLMScoreRequest,
+    ScoreResponse as vLLMScoreResponse,
+)
 from vllm.utils import random_uuid
 
 if TYPE_CHECKING:
@@ -59,15 +59,6 @@ class ErrorResponse(vLLMErrorResponse):
 class CompletionRequest(vLLMCompletionRequest):
     model_config = ConfigDict(arbitrary_types_allowed=True)
 
-    request_id: str = Field(
-        default_factory=lambda: f"{random_uuid()}",
-        description=(
-            "The request_id related to this request. If the caller does "
-            "not set it, a random_uuid will be generated. This id is used "
-            "through out the inference process and return in response."
-        ),
-    )
-
 
 class CompletionResponse(vLLMCompletionResponse):
     model_config = ConfigDict(arbitrary_types_allowed=True)
@@ -81,15 +72,6 @@ class CompletionStreamResponse(vLLMCompletionStreamResponse):
 class EmbeddingCompletionRequest(vLLMEmbeddingCompletionRequest):
     model_config = ConfigDict(arbitrary_types_allowed=True)
 
-    request_id: str = Field(
-        default_factory=lambda: f"{random_uuid()}",
-        description=(
-            "The request_id related to this request. If the caller does "
-            "not set it, a random_uuid will be generated. This id is used "
-            "through out the inference process and return in response."
-        ),
-    )
-
 
 class EmbeddingChatRequest(vLLMEmbeddingChatRequest):
     model_config = ConfigDict(arbitrary_types_allowed=True)
