@@ -18,7 +18,7 @@ from typing import (
     Union,
 )
 
-from fastapi import FastAPI, Form, HTTPException, status
+from fastapi import FastAPI, Form, HTTPException, Request, status
 from fastapi.middleware.cors import CORSMiddleware
 from starlette.responses import JSONResponse, Response, StreamingResponse
 
@@ -31,6 +31,7 @@ from ray.llm._internal.common.utils.lora_utils import (
 from ray.llm._internal.serve.constants import (
     DEFAULT_LLM_ROUTER_HTTP_TIMEOUT,
     DEFAULT_MAX_ONGOING_REQUESTS,
+    DEFAULT_MAX_TARGET_ONGOING_REQUESTS,
 )
 from ray.llm._internal.serve.core.configs.llm_config import LLMConfig
 from ray.llm._internal.serve.core.configs.openai_api_models import (
@@ -62,7 +63,7 @@ from ray.llm._internal.serve.core.ingress.middleware import (
     SetRequestIdMiddleware,
     add_exception_handling_middleware,
 )
-from ray.llm._internal.serve.core.protocol import DeploymentProtocol
+from ray.llm._internal.serve.core.protocol import DeploymentProtocol, RawRequestInfo
 from ray.llm._internal.serve.observability.logging import get_logger
 from ray.llm._internal.serve.observability.metrics.fast_api_metrics import (
     add_http_metrics_middleware,
@@ -87,6 +88,9 @@ T = TypeVar("T")
 
 DEFAULT_INGRESS_OPTIONS = {
     "max_ongoing_requests": DEFAULT_MAX_ONGOING_REQUESTS,
+    "autoscaling_config": {
+        "target_ongoing_requests": DEFAULT_MAX_TARGET_ONGOING_REQUESTS,
+    },
 }
 
 # These methods correspond to functions defined in the LLMEngine class in python/ray/llm/_internal/serve/deployments/llm/llm_engine.py
@@ -119,7 +123,7 @@ def _sanitize_chat_completion_request(
 
     TODO(seiji): Remove when we update to Pydantic v2.11+ with the fix.
     """
-    from vllm.transformers_utils.tokenizers.mistral import maybe_serialize_tool_calls
+    from vllm.tokenizers.mistral import maybe_serialize_tool_calls
 
     maybe_serialize_tool_calls(request)
 
@@ -237,9 +241,8 @@ def make_fastapi_ingress(
         class_dict[method_name] = decorated_method
 
     # Create new class with the decorated methods in its __dict__.
-    # IMPORTANT: We keep the same __name__ and __qualname__ as the original
-    # class so that make_fastapi_class_based_view can properly identify the routes
-    # (it checks if cls.__qualname__ is in route.endpoint.__qualname__).
+    # We keep the same __name__ and __qualname__ as the original class
+    # so that the new class properly represents the input class.
     new_cls = type(cls.__name__, (cls,), class_dict)
     new_cls.__qualname__ = cls.__qualname__
 
@@ -420,6 +423,29 @@ class OpenAiIngress(DeploymentProtocol):
 
         return self._configured_serve_handles[model_id]
 
+    async def _get_model_id(self, model: Optional[str]) -> str:
+        # Default to the only configured model if no model specified
+        if model is None:
+            if len(self._llm_configs) == 1:
+                model = next(iter(self._llm_configs.keys()))
+            else:
+                raise HTTPException(
+                    status.HTTP_400_BAD_REQUEST,
+                    "Model parameter is required when multiple models are configured. "
+                    f"Available models: {list(self._llm_configs.keys())}",
+                )
+
+        base_model_id = get_base_model_id(model)
+        if base_model_id not in self._llm_configs:
+            raise HTTPException(
+                status.HTTP_404_NOT_FOUND,
+                f'Got request for model "{model}". '
+                f'Could not find base model with ID "{base_model_id}".',
+            )
+
+        # Return original model ID so multiplexed routing works correctly.
+        return model
+
     async def _get_response(
         self,
         *,
@@ -431,6 +457,7 @@ class OpenAiIngress(DeploymentProtocol):
             ScoreRequest,
         ],
         call_method: str,
+        raw_request: Optional[Request] = None,
     ) -> AsyncGenerator[
         Union[
             LLMChatResponse,
@@ -442,23 +469,22 @@ class OpenAiIngress(DeploymentProtocol):
         None,
     ]:
         """Calls the model deployment and returns the stream."""
-        model: str = body.model
-        base_model_id = get_base_model_id(model)
-        if base_model_id not in self._llm_configs:
-            raise HTTPException(
-                status.HTTP_404_NOT_FOUND,
-                f'Got request for model "{model}". '
-                f'Could not find base model with ID "{base_model_id}".',
-            )
-
-        model_handle = self._get_configured_serve_handle(model)
+        model_id = await self._get_model_id(body.model)
+        model_handle = self._get_configured_serve_handle(model_id)
 
         # TODO(seiji): Remove when we update to Pydantic v2.11+ with the fix
         # for tool calling ValidatorIterator serialization issue.
         if isinstance(body, ChatCompletionRequest):
             body = _sanitize_chat_completion_request(body)
 
-        async for response in getattr(model_handle, call_method).remote(body):
+        # Convert Starlette request to serializable RawRequestInfo
+        raw_request_info: Optional[RawRequestInfo] = None
+        if raw_request is not None:
+            raw_request_info = RawRequestInfo.from_starlette_request(raw_request)
+
+        async for response in getattr(model_handle, call_method).remote(
+            body, raw_request_info
+        ):
             yield response
 
     async def model(self, model_id: str) -> Optional[ModelCard]:
@@ -526,11 +552,14 @@ class OpenAiIngress(DeploymentProtocol):
         self,
         body: Union[CompletionRequest, ChatCompletionRequest, TranscriptionRequest],
         call_method: str,
+        raw_request: Optional[Request] = None,
     ) -> Response:
 
         async with router_request_timeout(DEFAULT_LLM_ROUTER_HTTP_TIMEOUT):
 
-            gen = self._get_response(body=body, call_method=call_method)
+            gen = self._get_response(
+                body=body, call_method=call_method, raw_request=raw_request
+            )
 
             # In streaming with batching enabled, this first response can be a list of chunks.
             initial_response, gen = await _peek_at_generator(gen)
@@ -558,44 +587,50 @@ class OpenAiIngress(DeploymentProtocol):
                 openai_stream_generator, media_type="text/event-stream"
             )
 
-    async def completions(self, body: CompletionRequest) -> Response:
+    async def completions(self, body: CompletionRequest, request: Request) -> Response:
         """Given a prompt, the model will return one or more predicted completions,
         and can also return the probabilities of alternative tokens at each position.
 
         Args:
-            body: The CompletionRequest object.
+            body: The completion request.
+            request: The raw FastAPI request object.
 
         Returns:
             A response object with completions.
         """
         return await self._process_llm_request(
-            body, call_method=CallMethod.COMPLETIONS.value
+            body, call_method=CallMethod.COMPLETIONS.value, raw_request=request
         )
 
-    async def chat(self, body: ChatCompletionRequest) -> Response:
+    async def chat(self, body: ChatCompletionRequest, request: Request) -> Response:
         """Given a prompt, the model will return one or more predicted completions,
         and can also return the probabilities of alternative tokens at each position.
 
         Args:
-            body: The ChatCompletionRequest object.
+            body: The chat completion request.
+            request: The raw FastAPI request object.
 
         Returns:
             A response object with completions.
         """
+        return await self._process_llm_request(
+            body, call_method=CallMethod.CHAT.value, raw_request=request
+        )
 
-        return await self._process_llm_request(body, call_method=CallMethod.CHAT.value)
-
-    async def embeddings(self, body: EmbeddingRequest) -> Response:
+    async def embeddings(self, body: EmbeddingRequest, request: Request) -> Response:
         """Create embeddings for the provided input.
 
         Args:
-            body: The EmbeddingRequest object.
+            body: The embedding request.
+            request: The raw FastAPI request object.
 
         Returns:
             A response object with embeddings.
         """
         async with router_request_timeout(DEFAULT_LLM_ROUTER_HTTP_TIMEOUT):
-            results = self._get_response(body=body, call_method="embeddings")
+            results = self._get_response(
+                body=body, call_method="embeddings", raw_request=request
+            )
             result = await results.__anext__()
             if isinstance(result, ErrorResponse):
                 raise OpenAIHTTPException(
@@ -610,35 +645,39 @@ class OpenAiIngress(DeploymentProtocol):
     # Annotated[..., Form()] is wrapper that is used to handle multiple form data, which is how audio is sent in transcription requests.
     # vLLM implementation for handling transcription requests: https://github.com/vllm-project/vllm/blob/0825197bee8dea547f2ab25f48afd8aea0cd2578/vllm/entrypoints/openai/api_server.py#L839.
     async def transcriptions(
-        self, body: Annotated[TranscriptionRequest, Form()]
+        self, body: Annotated[TranscriptionRequest, Form()], request: Request
     ) -> Response:
         """Create transcription for the provided audio input.
 
         Args:
             body: The TranscriptionRequest object.
+            request: The raw FastAPI request object.
 
         Returns:
             A response object with transcriptions.
         """
 
         return await self._process_llm_request(
-            body, call_method=CallMethod.TRANSCRIPTIONS.value
+            body, call_method=CallMethod.TRANSCRIPTIONS.value, raw_request=request
         )
 
-    async def score(self, body: ScoreRequest) -> Response:
+    async def score(self, body: ScoreRequest, request: Request) -> Response:
         """Create scores for the provided text pairs.
 
         Note: This is a vLLM specific endpoint.
 
         Args:
             body: The score request containing input text pairs to score.
+            request: The raw FastAPI request object.
 
         Returns:
             A response object with scores.
         """
 
         async with router_request_timeout(DEFAULT_LLM_ROUTER_HTTP_TIMEOUT):
-            results = self._get_response(body=body, call_method="score")
+            results = self._get_response(
+                body=body, call_method="score", raw_request=request
+            )
             result = await results.__anext__()
             if isinstance(result, ErrorResponse):
                 raise OpenAIHTTPException(
