@@ -27,7 +27,7 @@ from ray.llm._internal.serve.core.configs.llm_config import (
     LLMConfig,
 )
 from ray.llm._internal.serve.core.engine.protocol import LLMEngine
-from ray.llm._internal.serve.core.protocol import LLMServerProtocol
+from ray.llm._internal.serve.core.protocol import LLMServerProtocol, RawRequestInfo
 from ray.llm._internal.serve.engines.vllm.vllm_engine import VLLMEngine
 from ray.llm._internal.serve.observability.logging import get_logger
 from ray.llm._internal.serve.observability.usage_telemetry.usage import (
@@ -293,6 +293,7 @@ class LLMServer(LLMServerProtocol):
         *,
         engine_method: str,
         batch_output_stream: bool = False,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Any, None]:
         """Run the engine method on the request + perform batching when stream=True.
 
@@ -300,6 +301,8 @@ class LLMServer(LLMServerProtocol):
             request: The request to run.
             engine_method: The method to call on the engine.
             batch_output_stream: Whether to batch the output stream.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
             An AsyncGenerator of the response. If stream is True and batching is enabled, then the generator will yield a list of streaming responses (strings of the format data: {response_json}\n\n). Otherwise, it will yield the non-streaming response from engine directly.
@@ -309,17 +312,19 @@ class LLMServer(LLMServerProtocol):
         await self._maybe_resolve_lora_from_multiplex()
 
         is_stream = hasattr(request, "stream") and request.stream
+        engine_stream = getattr(self.engine, engine_method)(request, raw_request_info)
+
         if is_stream and batch_output_stream:
-            stream = self._batch_output_stream(
-                getattr(self.engine, engine_method)(request)
-            )
+            stream = self._batch_output_stream(engine_stream)
         else:
-            stream = getattr(self.engine, engine_method)(request)
+            stream = engine_stream
 
         return stream
 
     async def chat(
-        self, request: "ChatCompletionRequest"
+        self,
+        request: "ChatCompletionRequest",
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[
         Union[List[Union[str, "ErrorResponse"]], "ChatCompletionResponse"], None
     ]:
@@ -327,18 +332,26 @@ class LLMServer(LLMServerProtocol):
 
         Args:
             request: A ChatCompletionRequest object.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
-            An AsyncGenerator of the response. If stream is True and batching is enabled, then the generator will yield a list of chat streaming responses (strings of the format data: {response_json}\n\n). Otherwise, it will yield the ChatCompletionResponse object directly.
+            An AsyncGenerator of the response. If stream is True and batching
+            is enabled, then the generator will yield a list of chat streaming
+            responses (strings of the format data: {response_json}\\n\\n).
+            Otherwise, it will yield the ChatCompletionResponse object directly.
         """
         return await self._run_request(
             request,
             engine_method="chat",
             batch_output_stream=True,
+            raw_request_info=raw_request_info,
         )
 
     async def completions(
-        self, request: "CompletionRequest"
+        self,
+        request: "CompletionRequest",
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[
         Union[List[Union[str, "ErrorResponse"]], "CompletionResponse"], None
     ]:
@@ -346,18 +359,26 @@ class LLMServer(LLMServerProtocol):
 
         Args:
             request: A CompletionRequest object.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
-            An AsyncGenerator of the response. If stream is True and batching is enabled, then the generator will yield a list of completion streaming responses (strings of the format data: {response_json}\n\n). Otherwise, it will yield the CompletionResponse object directly.
+            An AsyncGenerator of the response. If stream is True and batching
+            is enabled, then the generator will yield a list of completion
+            streaming responses (strings of the format data: {response_json}\\n\\n).
+            Otherwise, it will yield the CompletionResponse object directly.
         """
         return await self._run_request(
             request,
             engine_method="completions",
             batch_output_stream=True,
+            raw_request_info=raw_request_info,
         )
 
     async def embeddings(
-        self, request: "EmbeddingRequest"
+        self,
+        request: "EmbeddingRequest",
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[List["ErrorResponse"], "EmbeddingResponse"], None]:
         """Runs an embeddings request to the engine and returns the response.
 
@@ -365,6 +386,8 @@ class LLMServer(LLMServerProtocol):
 
         Args:
             request: An EmbeddingRequest object.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
             An AsyncGenerator over the EmbeddingResponse object.
@@ -374,10 +397,13 @@ class LLMServer(LLMServerProtocol):
             request,
             engine_method="embeddings",
             batch_output_stream=False,
+            raw_request_info=raw_request_info,
         )
 
     async def transcriptions(
-        self, request: "TranscriptionRequest"
+        self,
+        request: "TranscriptionRequest",
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[
         Union[List[Union[str, "ErrorResponse"]], "TranscriptionResponse"], None
     ]:
@@ -386,7 +412,9 @@ class LLMServer(LLMServerProtocol):
         Returns an AsyncGenerator over the TranscriptionResponse object. This is so that the caller can have a consistent interface across all the methods of chat, completions, embeddings and transcriptions.
 
         Args:
-            request: An TranscriptionRequest object.
+            request: A TranscriptionRequest object.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
             An AsyncGenerator over the TranscriptionResponse object.
@@ -395,10 +423,13 @@ class LLMServer(LLMServerProtocol):
             request,
             engine_method="transcriptions",
             batch_output_stream=True,
+            raw_request_info=raw_request_info,
         )
 
     async def score(
-        self, request: "ScoreRequest"
+        self,
+        request: "ScoreRequest",
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union["ScoreResponse", "ErrorResponse"], None]:
         """Runs a score request to the engine and returns the response.
 
@@ -406,13 +437,18 @@ class LLMServer(LLMServerProtocol):
 
         Args:
             request: A ScoreRequest object.
+            raw_request_info: Optional RawRequestInfo containing data from the original
+                HTTP request.
 
         Returns:
             An AsyncGenerator over the ScoreResponse object.
         """
         # NOTE: Score does not need batching, similar to embeddings.
         return await self._run_request(
-            request, engine_method="score", batch_output_stream=False
+            request,
+            engine_method="score",
+            batch_output_stream=False,
+            raw_request_info=raw_request_info,
         )
 
     async def check_health(self) -> None:
