@@ -1,4 +1,5 @@
 import argparse
+import inspect
 import os
 from typing import TYPE_CHECKING, AsyncGenerator, Optional, Tuple, Union
 
@@ -30,6 +31,7 @@ from ray.llm._internal.serve.core.configs.openai_api_models import (
     TranscriptionResponse,
 )
 from ray.llm._internal.serve.core.engine.protocol import LLMEngine
+from ray.llm._internal.serve.core.protocol import RawRequestInfo
 from ray.llm._internal.serve.engines.vllm.vllm_models import (
     VLLMEngineConfig,
 )
@@ -134,8 +136,7 @@ class VLLMEngine(LLMEngine):
             )
         from vllm import envs as vllm_envs
 
-        # TODO (Kourosh): Remove this after a few releases.
-        if not vllm_envs.VLLM_USE_V1:
+        if hasattr(vllm_envs, "VLLM_USE_V1") and not vllm_envs.VLLM_USE_V1:
             logger.error(
                 "vLLM v0 is fully deprecated. As a result in Ray Serve LLM only v1 is supported."
             )
@@ -196,17 +197,22 @@ class VLLMEngine(LLMEngine):
         state = State()
         # TODO (Kourosh): There might be some variables that needs protection?
         args = argparse.Namespace(
-            **vllm_frontend_args.__dict__,
-            **vllm_engine_args.__dict__,
+            **(vllm_frontend_args.__dict__ | vllm_engine_args.__dict__)
         )
 
-        await init_app_state(
-            engine_client=self._engine_client,
-            # TODO (ahao): remove vllm_config for vllm v1.12
-            vllm_config=vllm_engine_config,
-            state=state,
-            args=args,
-        )
+        if "vllm_config" in inspect.signature(init_app_state).parameters:
+            await init_app_state(
+                self._engine_client,
+                vllm_config=vllm_engine_config,
+                state=state,
+                args=args,
+            )
+        else:
+            await init_app_state(
+                self._engine_client,
+                state=state,
+                args=args,
+            )
 
         self._oai_models = state.openai_serving_models
         self._oai_serving_chat = state.openai_serving_chat
@@ -358,39 +364,19 @@ class VLLMEngine(LLMEngine):
         if isinstance(lora_request, VLLMErrorResponse):
             raise ValueError(f"Failed to load lora model: {lora_request.error.message}")
 
-    def _create_raw_request(
-        self,
-        request: Union[
-            CompletionRequest,
-            ChatCompletionRequest,
-            EmbeddingRequest,
-            TranscriptionRequest,
-            ScoreRequest,
-        ],
-        path: str,
-    ) -> Request:
-        scope = {
-            "type": "http",
-            "method": "POST",
-            "path": path,
-            "headers": [(b"x-request-id", getattr(request, "request_id", "").encode())],
-            "query_string": b"",
-        }
-        return Request(scope)
-
     async def chat(
-        self, request: ChatCompletionRequest
+        self,
+        request: ChatCompletionRequest,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[str, ChatCompletionResponse, ErrorResponse], None]:
         self._validate_openai_serving_chat()
 
-        # TODO (Kourosh): Remove when we upstream request_id attribute to vLLM.
-        # PR: https://github.com/vllm-project/vllm/pull/21009
-        # Create a fake starlette.Request object with the x-request-id header
-        # so that the create_chat_completion API can assign the request_id properly.
-        raw_request = self._create_raw_request(request, "/chat/completions")
-
+        raw_request: Optional[Request] = RawRequestInfo.to_starlette_request_optional(
+            raw_request_info
+        )
         chat_response = await self._oai_serving_chat.create_chat_completion(  # type: ignore[attr-defined]
-            request, raw_request=raw_request
+            request,
+            raw_request=raw_request,
         )
 
         if isinstance(chat_response, AsyncGenerator):
@@ -407,16 +393,15 @@ class VLLMEngine(LLMEngine):
                 yield ChatCompletionResponse(**chat_response.model_dump())
 
     async def completions(
-        self, request: CompletionRequest
+        self,
+        request: CompletionRequest,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[str, CompletionResponse, ErrorResponse], None]:
         self._validate_openai_serving_completion()
 
-        # TODO (Kourosh): Remove when we upstream request_id attribute to vLLM.
-        # PR: https://github.com/vllm-project/vllm/pull/21009
-        # Create a fake starlette.Request object with the x-request-id header
-        # so that the create_completion API can assign the request_id properly.
-        raw_request = self._create_raw_request(request, "/completions")
-
+        raw_request: Optional[Request] = RawRequestInfo.to_starlette_request_optional(
+            raw_request_info
+        )
         completion_response = await self._oai_serving_completion.create_completion(  # type: ignore[attr-defined]
             request,
             raw_request=raw_request,
@@ -438,17 +423,18 @@ class VLLMEngine(LLMEngine):
                 yield CompletionResponse(**completion_response.model_dump())
 
     async def embeddings(
-        self, request: EmbeddingRequest
+        self,
+        request: EmbeddingRequest,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[EmbeddingResponse, ErrorResponse], None]:
         self._validate_openai_serving_embedding()
 
-        # TODO (Kourosh): Remove when upstream is fixed to accept req_id.
-        # Create a fake starlette.Request object with the x-request-id header
-        # so that the create_embedding API can assign the request_id properly.
-        raw_request = self._create_raw_request(request, "/embeddings")
-
+        raw_request: Optional[Request] = RawRequestInfo.to_starlette_request_optional(
+            raw_request_info
+        )
         embedding_response = await self._oai_serving_embedding.create_embedding(  # type: ignore[attr-defined]
-            request, raw_request=raw_request
+            request,
+            raw_request=raw_request,
         )
 
         if isinstance(embedding_response, VLLMErrorResponse):
@@ -459,19 +445,18 @@ class VLLMEngine(LLMEngine):
             yield EmbeddingResponse(**embedding_response.model_dump())
 
     async def transcriptions(
-        self, request: TranscriptionRequest
+        self,
+        request: TranscriptionRequest,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[str, TranscriptionResponse, ErrorResponse], None]:
         self._validate_openai_serving_transcription()
 
-        # TODO (Kourosh): Remove when we upstream request_id attribute to vLLM.
-        # PR: https://github.com/vllm-project/vllm/pull/21009
-        # Create a fake starlette.Request object with the x-request-id header
-        # so that the create_transcription API can assign the request_id properly.
-        raw_request = self._create_raw_request(request, "/audio/transcriptions")
-
         # Extract audio data from the request file
         audio_data = await request.file.read()
 
+        raw_request: Optional[Request] = RawRequestInfo.to_starlette_request_optional(
+            raw_request_info
+        )
         transcription_response = await self._oai_serving_transcription.create_transcription(  # type: ignore[attr-defined]
             audio_data,
             request,
@@ -494,14 +479,18 @@ class VLLMEngine(LLMEngine):
                 yield TranscriptionResponse(**transcription_response.model_dump())
 
     async def score(
-        self, request: ScoreRequest
+        self,
+        request: ScoreRequest,
+        raw_request_info: Optional[RawRequestInfo] = None,
     ) -> AsyncGenerator[Union[ScoreResponse, ErrorResponse], None]:
         self._validate_openai_serving_scores()
 
-        raw_request = self._create_raw_request(request, "/score")
-
+        raw_request: Optional[Request] = RawRequestInfo.to_starlette_request_optional(
+            raw_request_info
+        )
         score_response = await self._oai_serving_scores.create_score(
-            request, raw_request=raw_request
+            request,
+            raw_request=raw_request,
         )
 
         if isinstance(score_response, VLLMErrorResponse):
