@@ -30,6 +30,7 @@ from ray.rllib.connectors.common import AddStatesFromEpisodesToBatch
 from ray.rllib.core import DEFAULT_MODULE_ID
 from ray.rllib.core.columns import Columns
 from ray.rllib.core.rl_module.rl_module import RLModuleSpec
+from ray.rllib.env import INPUT_ENV_SINGLE_SPACES
 from ray.rllib.execution.rollout_ops import synchronous_parallel_sample
 from ray.rllib.policy.sample_batch import SampleBatch
 from ray.rllib.utils import deep_update
@@ -82,11 +83,6 @@ class DreamerV3Config(AlgorithmConfig):
         algo = config.build()
         # algo.train()
         del algo
-
-    .. testoutput::
-        :hide:
-
-        ...
     """
 
     def __init__(self, algo_class=None):
@@ -576,6 +572,11 @@ class DreamerV3(Algorithm):
         replay_buffer_results = self.local_replay_buffer.get_metrics()
         self.metrics.aggregate([replay_buffer_results], key=REPLAY_BUFFER_RESULTS)
 
+        # Use self.spaces for the environment spaces of the env-runners
+        single_observation_space, single_action_space = self.spaces[
+            INPUT_ENV_SINGLE_SPACES
+        ]
+
         # Continue sampling batch_size_B x batch_length_T sized batches from the buffer
         # and using these to update our models (`LearnerGroup.update()`)
         # until the computed `training_ratio` is larger than the configured one, meaning
@@ -601,13 +602,11 @@ class DreamerV3(Algorithm):
                 replayed_steps = self.config.batch_size_B * self.config.batch_length_T
                 replayed_steps_this_iter += replayed_steps
 
-                if isinstance(
-                    self.env_runner.env.single_action_space, gym.spaces.Discrete
-                ):
+                if isinstance(single_action_space, gym.spaces.Discrete):
                     sample["actions_ints"] = sample[Columns.ACTIONS]
                     sample[Columns.ACTIONS] = one_hot(
                         sample["actions_ints"],
-                        depth=self.env_runner.env.single_action_space.n,
+                        depth=single_action_space.n,
                     )
 
                 # Perform the actual update via our learner group.
@@ -627,7 +626,9 @@ class DreamerV3(Algorithm):
                 self.metrics.aggregate(learner_results, key=LEARNER_RESULTS)
 
                 sub_iter += 1
-                self.metrics.log_value(NUM_GRAD_UPDATES_LIFETIME, 1, reduce="sum")
+                self.metrics.log_value(
+                    NUM_GRAD_UPDATES_LIFETIME, 1, reduce="lifetime_sum"
+                )
 
         # Log videos showing how the decoder produces observation predictions
         # from the posterior states.
@@ -640,7 +641,7 @@ class DreamerV3(Algorithm):
             batch_size_B=self.config.batch_size_B,
             batch_length_T=self.config.batch_length_T,
             symlog_obs=do_symlog_obs(
-                self.env_runner.env.single_observation_space,
+                single_observation_space,
                 self.config.symlog_obs,
             ),
             do_report=(
@@ -661,7 +662,7 @@ class DreamerV3(Algorithm):
             dreamed_T=self.config.horizon_H + 1,
             dreamer_model=self.env_runner.module.dreamer_model,
             symlog_obs=do_symlog_obs(
-                self.env_runner.env.single_observation_space,
+                single_observation_space,
                 self.config.symlog_obs,
             ),
             do_report=(
