@@ -55,7 +55,7 @@ class MARWILConfig(AlgorithmConfig):
         # Get the base path (to ray/rllib)
         base_path = Path(__file__).parents[2]
         # Get the path to the data in rllib folder.
-        data_path = base_path / "tests/data/cartpole/cartpole-v1_large"
+        data_path = base_path / "offline/tests/data/cartpole/cartpole-v1_large"
 
         config = MARWILConfig()
         # Enable the new API stack.
@@ -107,7 +107,7 @@ class MARWILConfig(AlgorithmConfig):
         # Get the base path (to ray/rllib)
         base_path = Path(__file__).parents[2]
         # Get the path to the data in rllib folder.
-        data_path = base_path / "tests/data/cartpole/cartpole-v1_large"
+        data_path = base_path / "offline/tests/data/cartpole/cartpole-v1_large"
 
         config = MARWILConfig()
         # Enable the new API stack.
