@@ -2,7 +2,7 @@
 from ray.rllib.algorithms.ppo.torch.default_ppo_torch_rl_module import (  # noqa
     DefaultPPOTorchRLModule as PPOTorchRLModule,
 )
-from ray.rllib.utils.deprecation import deprecation_warning
+from ray._common.deprecation import deprecation_warning
 
 
 deprecation_warning(
