@@ -223,14 +223,13 @@ class SACTorchLearner(DQNTorchLearner, SACLearner):
         if config.twin_q:
             total_loss += critic_twin_loss
 
-        # Log the TD-error with reduce=None, such that - in case we have n parallel
+        # Log the TD-error with reduce="item_series", such that - in case we have n parallel
         # Learners - we will re-concatenate the produced TD-error tensors to yield
         # a 1:1 representation of the original batch.
         self.metrics.log_value(
             key=(module_id, TD_ERROR_KEY),
             value=td_error,
-            reduce=None,
-            clear_on_reduce=True,
+            reduce="item_series",
         )
         # Log other important loss stats (reduce=mean (default), but with window=1
         # in order to keep them history free).
@@ -356,14 +355,13 @@ class SACTorchLearner(DQNTorchLearner, SACLearner):
             # TODO (simon): Check, if we need to multiply the critic_loss then with 0.5.
             total_loss += critic_twin_loss
 
-        # Log the TD-error with reduce=None, such that - in case we have n parallel
+        # Log the TD-error with reduce="item_series", such that - in case we have n parallel
         # Learners - we will re-concatenate the produced TD-error tensors to yield
         # a 1:1 representation of the original batch.
         self.metrics.log_value(
             key=(module_id, TD_ERROR_KEY),
             value=td_error,
-            reduce=None,
-            clear_on_reduce=True,
+            reduce="item_series",
         )
         # Log other important loss stats (reduce=mean (default), but with window=1
         # in order to keep them history free).
