@@ -112,9 +112,9 @@ from ray.rllib.examples.algorithms.classes.maml_lr_differentiable_rlm import (
 from ray.rllib.examples.algorithms.classes.maml_lr_meta_learner import (
     MAMLTorchMetaLearner,
 )
+from ray.rllib.examples.utils import add_rllib_example_script_args
 from ray.rllib.policy.sample_batch import MultiAgentBatch, SampleBatch
 from ray.rllib.utils.framework import try_import_torch
-from ray.rllib.utils.test_utils import add_rllib_example_script_args
 
 # Import torch.
 torch, _ = try_import_torch()
