@@ -26,7 +26,6 @@ from ray.rllib.utils.metrics import (
     NUM_AGENT_EPISODES_EVICTED,
     NUM_AGENT_EPISODES_EVICTED_LIFETIME,
     NUM_AGENT_EPISODES_PER_SAMPLE,
-    NUM_AGENT_EPISODES_STORED,
     NUM_AGENT_RESAMPLES,
     NUM_AGENT_STEPS_ADDED,
     NUM_AGENT_STEPS_ADDED_LIFETIME,
@@ -36,7 +35,6 @@ from ray.rllib.utils.metrics import (
     NUM_AGENT_STEPS_PER_SAMPLE_LIFETIME,
     NUM_AGENT_STEPS_SAMPLED,
     NUM_AGENT_STEPS_SAMPLED_LIFETIME,
-    NUM_AGENT_STEPS_STORED,
     NUM_ENV_STEPS_ADDED,
     NUM_ENV_STEPS_ADDED_LIFETIME,
     NUM_ENV_STEPS_EVICTED,
@@ -57,7 +55,6 @@ from ray.rllib.utils.metrics import (
     NUM_MODULE_EPISODES_EVICTED,
     NUM_MODULE_EPISODES_EVICTED_LIFETIME,
     NUM_MODULE_EPISODES_PER_SAMPLE,
-    NUM_MODULE_EPISODES_STORED,
     NUM_MODULE_RESAMPLES,
     NUM_MODULE_STEPS_ADDED,
     NUM_MODULE_STEPS_ADDED_LIFETIME,
@@ -176,11 +173,7 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
         self.rng = np.random.default_rng(seed=None)
 
         # Initialize the metrics.
-        self.metrics = MetricsLogger()
-        self._metrics_num_episodes_for_smoothing = metrics_num_episodes_for_smoothing
-
-        # Initialize the metrics.
-        self.metrics = MetricsLogger()
+        self.metrics = MetricsLogger(root=False)
         self._metrics_num_episodes_for_smoothing = metrics_num_episodes_for_smoothing
 
     @override(ReplayBufferInterface)
@@ -391,23 +384,21 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
             NUM_EPISODES_ADDED,
             num_episodes_added,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_EPISODES_ADDED_LIFETIME,
             num_episodes_added,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
         self.metrics.log_value(
             NUM_EPISODES_EVICTED,
             num_episodes_evicted,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_EPISODES_EVICTED_LIFETIME,
             num_episodes_evicted,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
         # Whole buffer step metrics.
         self.metrics.log_value(
@@ -420,23 +411,21 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
             NUM_ENV_STEPS_ADDED,
             num_env_steps_added,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_ADDED_LIFETIME,
             num_env_steps_added,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_EVICTED,
             num_env_steps_evicted,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_EVICTED_LIFETIME,
             num_env_steps_evicted,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
 
         # Log per-agent metrics.
@@ -447,12 +436,11 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_AGENT_EPISODES_ADDED, aid),
                 agent_to_num_episodes_added[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_EPISODES_ADDED_LIFETIME, aid),
                 agent_to_num_episodes_added[aid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             # Number of new agent steps added. Note, this metric could
             # be zero, too.
@@ -460,12 +448,11 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_AGENT_STEPS_ADDED, aid),
                 agent_to_num_steps_added[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_ADDED_LIFETIME, aid),
                 agent_to_num_steps_added[aid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
         for aid in agent_to_num_episodes_evicted:
             # Number of agent episodes evicted. Note, values could be zero.
@@ -473,72 +460,22 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_AGENT_EPISODES_EVICTED, aid),
                 agent_to_num_episodes_evicted[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_EPISODES_EVICTED_LIFETIME, aid),
                 agent_to_num_episodes_evicted[aid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             # Number of agent steps evicted. Note, values could be zero.
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_EVICTED, aid),
                 agent_to_num_steps_evicted[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_EVICTED_LIFETIME, aid),
                 agent_to_num_steps_evicted[aid],
-                reduce="sum",
-            )
-        # Note, we need to loop through the metrics here to receive
-        # metrics for all agents (not only the ones that steps).
-        for aid in self.metrics.stats[NUM_AGENT_STEPS_ADDED_LIFETIME]:
-            # Add default metrics for evicted episodes if not existent.
-            if aid not in agent_to_num_episodes_evicted:
-                self.metrics.log_value(
-                    (NUM_AGENT_EPISODES_EVICTED, aid),
-                    0,
-                    reduce="sum",
-                    clear_on_reduce=True,
-                )
-                self.metrics.log_value(
-                    (NUM_AGENT_EPISODES_EVICTED_LIFETIME, aid),
-                    0,
-                    reduce="sum",
-                )
-                self.metrics.log_value(
-                    (NUM_AGENT_STEPS_EVICTED, aid),
-                    0,
-                    reduce="sum",
-                    clear_on_reduce=True,
-                )
-                self.metrics.log_value(
-                    (NUM_AGENT_STEPS_EVICTED_LIFETIME, aid),
-                    0,
-                    reduce="sum",
-                )
-            # Number of episodes in the buffer.
-            agent_episodes_stored = self.metrics.peek(
-                (NUM_AGENT_EPISODES_ADDED_LIFETIME, aid)
-            ) - self.metrics.peek((NUM_AGENT_EPISODES_EVICTED_LIFETIME, aid))
-            self.metrics.log_value(
-                (NUM_AGENT_EPISODES_STORED, aid),
-                agent_episodes_stored,
-                reduce="mean",
-                window=self._metrics_num_episodes_for_smoothing,
-            )
-            # Number of agent steps in the buffer.
-            agent_steps_stored = self.metrics.peek(
-                (NUM_AGENT_STEPS_ADDED_LIFETIME, aid)
-            ) - self.metrics.peek((NUM_AGENT_EPISODES_EVICTED_LIFETIME, aid))
-
-            self.metrics.log_value(
-                (NUM_AGENT_STEPS_STORED, aid),
-                agent_steps_stored,
-                reduce="mean",
-                window=self._metrics_num_episodes_for_smoothing,
+                reduce="lifetime_sum",
             )
 
         # Log per-module metrics.
@@ -549,12 +486,11 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_MODULE_EPISODES_ADDED, mid),
                 module_to_num_episodes_added[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_EPISODES_ADDED_LIFETIME, mid),
                 module_to_num_episodes_added[mid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             # Number of new module steps added. Note, this metric could
             # be zero, too.
@@ -562,12 +498,11 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_MODULE_STEPS_ADDED, mid),
                 module_to_num_steps_added[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_ADDED_LIFETIME, mid),
                 module_to_num_steps_added[mid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
         for mid in module_to_num_episodes_evicted:
             # Number of module episodes evicted. Note, values could be zero.
@@ -575,60 +510,22 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_MODULE_EPISODES_EVICTED, mid),
                 module_to_num_episodes_evicted[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_EPISODES_EVICTED_LIFETIME, mid),
                 module_to_num_episodes_evicted[mid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             # Number of module steps evicted. Note, values could be zero.
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_EVICTED, mid),
                 module_to_num_steps_evicted[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_EVICTED_LIFETIME, mid),
                 module_to_num_steps_evicted[mid],
-                reduce="sum",
-            )
-        # Note, we need to loop through the metrics here to receive
-        # metrics for all agents (not only the ones that steps).
-        for mid in self.metrics.stats[NUM_MODULE_STEPS_ADDED_LIFETIME]:
-            # Number of episodes in the buffer.
-            if mid not in module_to_num_episodes_evicted:
-                self.metrics.log_value(
-                    (NUM_MODULE_EPISODES_EVICTED, mid),
-                    0,
-                    reduce="sum",
-                    clear_on_reduce=True,
-                )
-                self.metrics.log_value(
-                    (NUM_MODULE_EPISODES_EVICTED_LIFETIME, mid),
-                    0,
-                    reduce="sum",
-                )
-                self.metrics.log_value(
-                    (NUM_MODULE_STEPS_EVICTED, mid),
-                    0,
-                    reduce="sum",
-                    clear_on_reduce=True,
-                )
-                self.metrics.log_value(
-                    (NUM_MODULE_STEPS_EVICTED_LIFETIME, mid),
-                    0,
-                    reduce="sum",
-                )
-            module_episodes_stored = self.metrics.peek(
-                (NUM_MODULE_EPISODES_ADDED_LIFETIME, mid)
-            ) - self.metrics.peek((NUM_MODULE_EPISODES_EVICTED_LIFETIME, mid))
-            self.metrics.log_value(
-                (NUM_MODULE_EPISODES_STORED, mid),
-                module_episodes_stored,
-                reduce="mean",
-                window=self._metrics_num_episodes_for_smoothing,
+                reduce="lifetime_sum",
             )
 
     @override(ReplayBufferInterface)
@@ -1239,34 +1136,30 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
             NUM_EPISODES_PER_SAMPLE,
             num_episodes_per_sample,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_PER_SAMPLE,
             num_env_steps_per_sample,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_PER_SAMPLE_LIFETIME,
             num_env_steps_per_sample,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_SAMPLED,
             num_env_steps_sampled,
             reduce="sum",
-            clear_on_reduce=True,
         )
         self.metrics.log_value(
             NUM_ENV_STEPS_SAMPLED_LIFETIME,
             num_env_steps_sampled,
-            reduce="sum",
+            reduce="lifetime_sum",
         )
         self.metrics.log_value(
             ENV_STEP_UTILIZATION,
-            self.metrics.peek(NUM_ENV_STEPS_PER_SAMPLE_LIFETIME)
-            / self.metrics.peek(NUM_ENV_STEPS_SAMPLED_LIFETIME),
+            num_env_steps_per_sample / num_env_steps_sampled,
             reduce="mean",
             window=self._metrics_num_episodes_for_smoothing,
         )
@@ -1296,7 +1189,6 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
             NUM_RESAMPLES,
             num_resamples,
             reduce="sum",
-            clear_on_reduce=True,
         )
 
         # Add per-agent metrics.
@@ -1305,36 +1197,32 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_AGENT_STEPS_SAMPLED, aid),
                 agent_to_num_steps_sampled[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             # TODO (simon): Check, if we can then deprecate
             # self.sampled_timesteps.
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_SAMPLED_LIFETIME, aid),
                 agent_to_num_steps_sampled[aid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             self.metrics.log_value(
                 (NUM_AGENT_EPISODES_PER_SAMPLE, aid),
                 agent_to_num_episodes_per_sample[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_PER_SAMPLE, aid),
                 agent_to_num_steps_per_sample[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_AGENT_STEPS_PER_SAMPLE_LIFETIME, aid),
                 agent_to_num_steps_per_sample[aid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             self.metrics.log_value(
                 (AGENT_STEP_UTILIZATION, aid),
-                self.metrics.peek((NUM_AGENT_STEPS_PER_SAMPLE_LIFETIME, aid))
-                / self.metrics.peek((NUM_AGENT_STEPS_SAMPLED_LIFETIME, aid)),
+                agent_to_num_steps_per_sample[aid] / agent_to_num_steps_sampled[aid],
                 reduce="mean",
                 window=self._metrics_num_episodes_for_smoothing,
             )
@@ -1342,7 +1230,6 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_AGENT_RESAMPLES, aid),
                 agent_to_num_resamples[aid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
         # Add per-module metrics.
         for mid in module_to_num_steps_sampled:
@@ -1350,36 +1237,32 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_MODULE_STEPS_SAMPLED, mid),
                 module_to_num_steps_sampled[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             # TODO (simon): Check, if we can then deprecate
             # self.sampled_timesteps.
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_SAMPLED_LIFETIME, mid),
                 module_to_num_steps_sampled[mid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             self.metrics.log_value(
                 (NUM_MODULE_EPISODES_PER_SAMPLE, mid),
                 module_to_num_episodes_per_sample[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_PER_SAMPLE, mid),
                 module_to_num_steps_per_sample[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
             self.metrics.log_value(
                 (NUM_MODULE_STEPS_PER_SAMPLE_LIFETIME, mid),
                 module_to_num_steps_per_sample[mid],
-                reduce="sum",
+                reduce="lifetime_sum",
             )
             self.metrics.log_value(
                 (MODULE_STEP_UTILIZATION, mid),
-                self.metrics.peek((NUM_MODULE_STEPS_PER_SAMPLE_LIFETIME, mid))
-                / self.metrics.peek((NUM_MODULE_STEPS_SAMPLED_LIFETIME, mid)),
+                module_to_num_steps_per_sample[mid] / module_to_num_steps_sampled[mid],
                 reduce="mean",
                 window=self._metrics_num_episodes_for_smoothing,
             )
@@ -1387,7 +1270,6 @@ class EpisodeReplayBuffer(ReplayBufferInterface):
                 (NUM_MODULE_RESAMPLES, mid),
                 module_to_num_resamples[mid],
                 reduce="sum",
-                clear_on_reduce=True,
             )
 
     # TODO (simon): Check, if we can instead peek into the metrics
