@@ -15,8 +15,8 @@ from packaging import version
 from starlette.types import Receive
 
 import ray
+from ray._common.filters import CoreContextFilter
 from ray._common.utils import get_or_create_event_loop
-from ray._private.ray_logging.filters import CoreContextFilter
 from ray.serve._private.common import (
     DeploymentID,
     EndpointInfo,
@@ -375,14 +375,17 @@ class GenericProxy(ABC):
                 if version.parse(starlette.__version__) < version.parse("0.33.0"):
                     proxy_request.set_path(route_path.replace(route_prefix, "", 1))
 
-            # NOTE(edoakes): we use the route_prefix instead of the full HTTP path
-            # for logs & metrics to avoid high cardinality.
-            # See: https://github.com/ray-project/ray/issues/47999
-            logs_and_metrics_route = (
-                route_prefix
-                if self.protocol == RequestProtocol.HTTP
-                else handle.deployment_id.app_name
-            )
+            # NOTE(abrar): we try to match to a specific route pattern (e.g., /api/{user_id})
+            # for logs & metrics when available. If no pattern matches, we fall back to the
+            # route_prefix to avoid high cardinality.
+            # See: https://github.com/ray-project/ray/issues/47999 and
+            # https://github.com/ray-project/ray/issues/52212
+            if self.protocol == RequestProtocol.HTTP:
+                logs_and_metrics_route = self.proxy_router.match_route_pattern(
+                    route_prefix, proxy_request.scope
+                )
+            else:
+                logs_and_metrics_route = handle.deployment_id.app_name
             internal_request_id = generate_request_id()
             handle, request_id = self.setup_request_context_and_handle(
                 app_name=handle.deployment_id.app_name,
@@ -1016,8 +1019,138 @@ class HTTPProxy(GenericProxy):
         yield status
 
 
+class ProxyActorInterface(ABC):
+    """Abstract interface for proxy actors in Ray Serve.
+
+    This interface defines the contract that all proxy actor implementations must follow,
+    allowing for different proxy backends (Ray HTTP/gRPC proxies, HAProxy, etc.).
+    """
+
+    def __init__(
+        self,
+        *,
+        node_id: NodeId,
+        node_ip_address: str,
+        logging_config: LoggingConfig,
+        log_buffer_size: int = RAY_SERVE_REQUEST_PATH_LOG_BUFFER_SIZE,
+    ):
+        """Initialize the proxy actor.
+
+        Args:
+            node_id: ID of the node this proxy is running on
+            node_ip_address: IP address of the node
+            logging_config: Logging configuration
+            log_buffer_size: Size of the log buffer
+        """
+        self._node_id = node_id
+        self._node_ip_address = node_ip_address
+        self._logging_config = logging_config
+        self._log_buffer_size = log_buffer_size
+
+        self._update_logging_config(logging_config)
+
+    @abstractmethod
+    async def ready(self) -> str:
+        """Blocks until the proxy is ready to serve requests.
+
+        Returns:
+            JSON-serialized metadata containing proxy information (worker ID, log file path, etc.)
+        """
+        pass
+
+    @abstractmethod
+    async def serving(self, wait_for_applications_running: bool = True) -> None:
+        """Wait for the proxy to be ready to serve requests.
+
+        Args:
+            wait_for_applications_running: Whether to wait for the applications to be running
+
+        Returns:
+            None
+        """
+        pass
+
+    @abstractmethod
+    async def update_draining(
+        self, draining: bool, _after: Optional[Any] = None
+    ) -> None:
+        """Update the draining status of the proxy.
+
+        Args:
+            draining: Whether the proxy should be draining
+            _after: Optional ObjectRef for scheduling dependency
+        """
+        pass
+
+    @abstractmethod
+    async def is_drained(self, _after: Optional[Any] = None) -> bool:
+        """Check whether the proxy is drained.
+
+        Args:
+            _after: Optional ObjectRef for scheduling dependency
+
+        Returns:
+            True if the proxy is drained, False otherwise
+        """
+        pass
+
+    @abstractmethod
+    async def check_health(self) -> bool:
+        """Check the health of the proxy.
+
+        Returns:
+            True if the proxy is healthy, False otherwise
+        """
+        pass
+
+    @abstractmethod
+    def pong(self) -> str:
+        """Respond to ping from replicas.
+
+        Returns:
+            A response string
+        """
+        pass
+
+    @abstractmethod
+    async def receive_asgi_messages(self, request_metadata: RequestMetadata) -> bytes:
+        """Handle ASGI messages for HTTP requests.
+
+        Args:
+            request_metadata: Metadata about the request
+
+        Returns:
+            Serialized ASGI messages
+        """
+        pass
+
+    # Testing and debugging methods
+    @abstractmethod
+    def _get_http_options(self) -> HTTPOptions:
+        """Get HTTP options used by the proxy."""
+        pass
+
+    @abstractmethod
+    def _get_logging_config(self) -> Optional[str]:
+        """Get the file path for the logger (for testing purposes)."""
+        pass
+
+    @abstractmethod
+    def _dump_ingress_replicas_for_testing(self, route: str) -> Set:
+        """Get replicas for a route (for testing)."""
+        pass
+
+    def _update_logging_config(self, logging_config: LoggingConfig):
+        configure_component_logger(
+            component_name="proxy",
+            component_id=self._node_ip_address,
+            logging_config=logging_config,
+            buffer_size=self._log_buffer_size,
+        )
+
+
 @ray.remote(num_cpus=0)
-class ProxyActor:
+class ProxyActor(ProxyActorInterface):
     def __init__(
         self,
         http_options: HTTPOptions,
@@ -1028,12 +1161,15 @@ class ProxyActor:
         logging_config: LoggingConfig,
         long_poll_client: Optional[LongPollClient] = None,
     ):  # noqa: F821
-        self._node_id = node_id
-        self._node_ip_address = node_ip_address
-        self._http_options = configure_http_middlewares(http_options)
-        self._grpc_options = grpc_options
-        grpc_enabled = is_grpc_enabled(self._grpc_options)
+        super().__init__(
+            node_id=node_id,
+            node_ip_address=node_ip_address,
+            logging_config=logging_config,
+        )
 
+        self._grpc_options = grpc_options
+        self._http_options = configure_http_middlewares(http_options)
+        grpc_enabled = is_grpc_enabled(self._grpc_options)
         event_loop = get_or_create_event_loop()
         self.long_poll_client = long_poll_client or LongPollClient(
             ray.get_actor(SERVE_CONTROLLER_NAME, namespace=SERVE_NAMESPACE),
@@ -1044,13 +1180,6 @@ class ProxyActor:
             call_in_event_loop=event_loop,
         )
 
-        configure_component_logger(
-            component_name="proxy",
-            component_id=node_ip_address,
-            logging_config=logging_config,
-            buffer_size=RAY_SERVE_REQUEST_PATH_LOG_BUFFER_SIZE,
-        )
-
         startup_msg = f"Proxy starting on node {self._node_id} (HTTP port: {self._http_options.port}"
         if grpc_enabled:
             startup_msg += f", gRPC port: {self._grpc_options.port})."
@@ -1151,13 +1280,6 @@ class ProxyActor:
     def _update_routes_in_proxies(self, endpoints: Dict[DeploymentID, EndpointInfo]):
         self.proxy_router.update_routes(endpoints)
 
-    def _update_logging_config(self, logging_config: LoggingConfig):
-        configure_component_logger(
-            component_name="proxy",
-            component_id=self._node_ip_address,
-            logging_config=logging_config,
-        )
-
     def _get_logging_config(self) -> Tuple:
         """Get the logging configuration (for testing purposes)."""
         log_file_path = None
@@ -1201,6 +1323,10 @@ class ProxyActor:
             ]
         )
 
+    async def serving(self, wait_for_applications_running: bool = True) -> None:
+        """Wait for the proxy to be ready to serve requests."""
+        return
+
     async def update_draining(self, draining: bool, _after: Optional[Any] = None):
         """Update the draining status of the HTTP and gRPC proxies.
 
@@ -1223,12 +1349,13 @@ class ProxyActor:
             self.grpc_proxy is None or self.grpc_proxy.is_drained()
         )
 
-    async def check_health(self):
+    async def check_health(self) -> bool:
         """No-op method to check on the health of the HTTP Proxy.
 
         Make sure the async event loop is not blocked.
         """
         logger.debug("Received health check.", extra={"log_to_stderr": False})
+        return True
 
     def pong(self):
         """Called by the replica to initialize its handle to the proxy."""
