@@ -98,7 +98,6 @@ from ray.serve._private.metrics_utils import InMemoryMetricsStore, MetricsPusher
 from ray.serve._private.task_consumer import TaskConsumerWrapper
 from ray.serve._private.thirdparty.get_asgi_route_name import (
     extract_route_patterns,
-    get_asgi_route_name,
 )
 from ray.serve._private.usage import ServeUsageTag
 from ray.serve._private.utils import (
@@ -247,8 +246,17 @@ class ReplicaMetricsManager:
 
         self.record_autoscaling_stats_failed_counter = metrics.Counter(
             "serve_record_autoscaling_stats_failed",
+            tag_keys=("exception_name",),
             description="The number of errored record_autoscaling_stats invocations.",
-            tag_keys=("app_name", "deployment_name", "replica_id", "exception_name"),
+        )
+
+        self.user_autoscaling_stats_latency_tracker = metrics.Histogram(
+            "serve_user_autoscaling_stats_latency_ms",
+            description=(
+                "Time taken to execute the user-defined autoscaling stats function "
+                "in milliseconds."
+            ),
+            boundaries=REQUEST_LATENCY_BUCKETS_MS,
         )
 
         self.set_autoscaling_config(autoscaling_config)
@@ -427,10 +435,13 @@ class ReplicaMetricsManager:
         self,
     ) -> Optional[Dict[str, Union[int, float]]]:
         try:
+            start_time = time.time()
             res = await asyncio.wait_for(
                 self._record_autoscaling_stats_fn(),
                 timeout=RAY_SERVE_RECORD_AUTOSCALING_STATS_TIMEOUT_S,
             )
+            latency_ms = (time.time() - start_time) * 1000
+            self.user_autoscaling_stats_latency_tracker.observe(latency_ms)
 
             # Perform validation only first call
             if not self._checked_custom_metrics:
@@ -456,27 +467,17 @@ class ReplicaMetricsManager:
                 self._checked_custom_metrics = True
 
             return res
-        except asyncio.TimeoutError as timeout_err:
+        except asyncio.TimeoutError as e:
             logger.error(
                 f"Replica autoscaling stats timed out after {RAY_SERVE_RECORD_AUTOSCALING_STATS_TIMEOUT_S}s."
             )
             self.record_autoscaling_stats_failed_counter.inc(
-                tags={
-                    "app_name": self._deployment_id.app_name,
-                    "deployment_name": self._deployment_id.name,
-                    "replica_id": self._replica_id.unique_id,
-                    "exception_name": timeout_err.__class__.__name__,
-                }
+                tags={"exception_name": e.__class__.__name__}
             )
-        except Exception as err:
-            logger.error(f"Replica autoscaling stats failed. {err}")
+        except Exception as e:
+            logger.error(f"Replica autoscaling stats failed. {e}")
             self.record_autoscaling_stats_failed_counter.inc(
-                tags={
-                    "app_name": self._deployment_id.app_name,
-                    "deployment_name": self._deployment_id.name,
-                    "replica_id": self._replica_id.unique_id,
-                    "exception_name": err.__class__.__name__,
-                }
+                tags={"exception_name": e.__class__.__name__}
             )
         return None
 
@@ -511,7 +512,6 @@ class ReplicaBase(ABC):
         version: DeploymentVersion,
         ingress: bool,
         route_prefix: str,
-        rank: ReplicaRank,
     ):
         self._version = version
         self._replica_id = replica_id
@@ -562,7 +562,7 @@ class ReplicaBase(ABC):
 
         # Set metadata for logs and metrics.
         # servable_object will be populated in `initialize_and_get_metadata`.
-        self._set_internal_replica_context(servable_object=None, rank=rank)
+        self._set_internal_replica_context(servable_object=None, rank=None)
 
         self._metrics_manager = create_replica_metrics_manager(
             replica_id=replica_id,
@@ -576,7 +576,7 @@ class ReplicaBase(ABC):
         self._http_port: Optional[int] = None
         self._grpc_port: Optional[int] = None
 
-        self._rank = rank
+        self._rank: Optional[ReplicaRank] = None
 
     @property
     def max_ongoing_requests(self) -> int:
@@ -725,36 +725,6 @@ class ReplicaBase(ABC):
         # A new request can be accepted if the semaphore is currently unlocked.
         return not self._semaphore.locked()
 
-    def _maybe_get_http_route(
-        self, request_metadata: RequestMetadata, request_args: Tuple[Any]
-    ) -> Optional[str]:
-        """Get the matched route string for ASGI apps to be used in logs & metrics.
-
-        If this replica does not wrap an ASGI app or there is no matching for the
-        request, returns the existing route from the request metadata.
-        """
-        route = request_metadata.route
-        if self._user_callable_asgi_app is not None:
-            req: StreamingHTTPRequest = request_args[0]
-            try:
-                matched_route = get_asgi_route_name(
-                    self._user_callable_asgi_app, req.asgi_scope
-                )
-            except Exception:
-                matched_route = None
-                logger.exception(
-                    "Failed unexpectedly trying to get route name for request. "
-                    "Routes in metric tags and log messages may be inaccurate. "
-                    "Please file a GitHub issue containing this traceback."
-                )
-
-            # If there is no match in the ASGI app, don't overwrite the route_prefix
-            # from the proxy.
-            if matched_route is not None:
-                route = matched_route
-
-        return route
-
     @contextmanager
     def _handle_errors_and_metrics(
         self, request_metadata: RequestMetadata
@@ -841,9 +811,6 @@ class ReplicaBase(ABC):
             )
 
             request_metadata._http_method = scope.get("method", "WS")
-            request_metadata.route = self._maybe_get_http_route(
-                request_metadata, request_args
-            )
 
             request_args = (scope, receive)
         elif request_metadata.is_grpc_request:
@@ -951,7 +918,14 @@ class ReplicaBase(ABC):
     async def _on_initialized(self):
         raise NotImplementedError
 
-    async def initialize(self, deployment_config: DeploymentConfig):
+    async def initialize(
+        self, deployment_config: Optional[DeploymentConfig], rank: Optional[ReplicaRank]
+    ):
+        if rank is not None:
+            self._rank = rank
+            self._set_internal_replica_context(
+                servable_object=self._user_callable_wrapper.user_callable, rank=rank
+            )
         try:
             # Ensure that initialization is only performed once.
             # When controller restarts, it will call this method again.
@@ -982,7 +956,7 @@ class ReplicaBase(ABC):
                             record_autoscaling_stats_fn=self._user_callable_wrapper.call_record_autoscaling_stats,
                         )
 
-                if deployment_config:
+                if deployment_config is not None:
                     await self._user_callable_wrapper.set_sync_method_threadpool_limit(
                         deployment_config.max_ongoing_requests
                     )
@@ -1227,7 +1201,6 @@ class ReplicaActor:
         version: DeploymentVersion,
         ingress: bool,
         route_prefix: str,
-        rank: ReplicaRank,
     ):
         deployment_config = DeploymentConfig.from_proto_bytes(
             deployment_config_proto_bytes
@@ -1244,7 +1217,6 @@ class ReplicaActor:
             version=version,
             ingress=ingress,
             route_prefix=route_prefix,
-            rank=rank,
         )
 
     def push_proxy_handle(self, handle: ActorHandle):
@@ -1287,7 +1259,7 @@ class ReplicaActor:
         return self._replica_impl.list_outbound_deployments()
 
     async def initialize_and_get_metadata(
-        self, deployment_config: DeploymentConfig = None, _after: Optional[Any] = None
+        self, deployment_config: DeploymentConfig = None, rank: ReplicaRank = None
     ) -> ReplicaMetadata:
         """Handles initializing the replica.
 
@@ -1300,7 +1272,7 @@ class ReplicaActor:
         """
         # Unused `_after` argument is for scheduling: passing an ObjectRef
         # allows delaying this call until after the `_after` call has returned.
-        await self._replica_impl.initialize(deployment_config)
+        await self._replica_impl.initialize(deployment_config, rank)
         return self._replica_impl.get_metadata()
 
     async def check_health(self):
