@@ -27,9 +27,16 @@ from typing import (
 from ray import serve
 from ray._common.signature import extract_signature, flatten_args, recover_args
 from ray._common.utils import get_or_create_event_loop
-from ray.serve._private.constants import SERVE_LOGGER_NAME
+from ray.serve._private.constants import (
+    BATCH_EXECUTION_TIME_BUCKETS_MS,
+    BATCH_SIZE_BUCKETS,
+    BATCH_UTILIZATION_BUCKETS_PERCENT,
+    BATCH_WAIT_TIME_BUCKETS_MS,
+    SERVE_LOGGER_NAME,
+)
 from ray.serve._private.utils import extract_self_if_method_call
 from ray.serve.exceptions import RayServeException
+from ray.serve.metrics import Counter, Gauge, Histogram
 from ray.util.annotations import PublicAPI
 
 logger = logging.getLogger(SERVE_LOGGER_NAME)
@@ -109,6 +116,7 @@ class _BatchQueue:
         batch_wait_timeout_s: float,
         max_concurrent_batches: int,
         handle_batch_func: Optional[Callable] = None,
+        batch_size_fn: Optional[Callable[[List], int]] = None,
     ) -> None:
         """Async queue that accepts individual items and returns batches.
 
@@ -128,11 +136,18 @@ class _BatchQueue:
             max_concurrent_batches: max number of batches to run concurrently.
             handle_batch_func(Optional[Callable]): callback to run in the
                 background to handle batches if provided.
+            batch_size_fn(Optional[Callable[[List], int]]): optional function to
+                compute the effective batch size. If None, uses len(batch).
+                The function takes a list of requests and returns an integer
+                representing the batch size. This is useful for batching based
+                on custom metrics such as total nodes in graphs, total tokens
+                in sequences, etc.
         """
         self.queue: asyncio.Queue[_SingleRequest] = asyncio.Queue()
         self.max_batch_size = max_batch_size
         self.batch_wait_timeout_s = batch_wait_timeout_s
         self.max_concurrent_batches = max_concurrent_batches
+        self.batch_size_fn = batch_size_fn
         self.semaphore = asyncio.Semaphore(max_concurrent_batches)
         self.requests_available_event = asyncio.Event()
         self.tasks: Set[asyncio.Task] = set()
@@ -140,6 +155,46 @@ class _BatchQueue:
         # Used for observability.
         self.curr_iteration_start_times: Dict[asyncio.Task, float] = {}
 
+        # Initialize batching metrics.
+        self._batch_wait_time_histogram = Histogram(
+            "serve_batch_wait_time_ms",
+            description="Time requests waited for batch to fill (in milliseconds).",
+            boundaries=BATCH_WAIT_TIME_BUCKETS_MS,
+            tag_keys=("function_name",),
+        )
+        self._batch_execution_time_histogram = Histogram(
+            "serve_batch_execution_time_ms",
+            description="Time to execute the batch function (in milliseconds).",
+            boundaries=BATCH_EXECUTION_TIME_BUCKETS_MS,
+            tag_keys=("function_name",),
+        )
+        self._batch_queue_length_gauge = Gauge(
+            "serve_batch_queue_length",
+            description="Number of requests waiting in the batch queue.",
+            tag_keys=("function_name",),
+        )
+        self._batch_utilization_histogram = Histogram(
+            "serve_batch_utilization_percent",
+            description="Batch utilization as percentage (actual_batch_size / max_batch_size * 100).",
+            boundaries=BATCH_UTILIZATION_BUCKETS_PERCENT,
+            tag_keys=("function_name",),
+        )
+        self._batch_size_histogram = Histogram(
+            "serve_actual_batch_size",
+            description="The actual number of requests in each batch.",
+            boundaries=BATCH_SIZE_BUCKETS,
+            tag_keys=("function_name",),
+        )
+        self._batches_processed_counter = Counter(
+            "serve_batches_processed",
+            description="Counter of batches executed.",
+            tag_keys=("function_name",),
+        )
+
+        self._function_name = (
+            handle_batch_func.__name__ if handle_batch_func is not None else "unknown"
+        )
+
         self._handle_batch_task = None
         self._loop = get_or_create_event_loop()
         if handle_batch_func is not None:
@@ -174,27 +229,65 @@ class _BatchQueue:
         self.queue.put_nowait(request)
         self.requests_available_event.set()
 
-    async def wait_for_batch(self) -> List[_SingleRequest]:
+    def _compute_batch_size(self, batch: List[_SingleRequest]) -> int:
+        """Compute the effective batch size using batch_size_fn or len()."""
+        if self.batch_size_fn is None:
+            return len(batch)
+
+        # Extract the actual data items from requests to pass to batch_size_fn.
+        # We need to reconstruct the original arguments from flattened_args.
+        items = []
+        for request in batch:
+            # Recover the original arguments from flattened format
+            args, kwargs = recover_args(request.flattened_args)
+            # The batch function expects a single positional argument (the item)
+            # after 'self' has been extracted (if it was a method)
+            items.append(args[0])
+
+        return self.batch_size_fn(items)
+
+    async def wait_for_batch(self) -> Tuple[List[_SingleRequest], int]:
         """Wait for batch respecting self.max_batch_size and self.timeout_s.
 
-        Returns a batch of up to self.max_batch_size items. Waits for up to
-        to self.timeout_s after receiving the first request that will be in
-        the next batch. After the timeout, returns as many items as are ready.
+        Returns a tuple of (batch, computed_batch_size) where batch contains
+        up to self.max_batch_size items. Waits for up to self.timeout_s after
+        receiving the first request that will be in the next batch. After the
+        timeout, returns as many items as are ready.
 
         Always returns a batch with at least one item - will block
         indefinitely until an item comes in.
         """
 
         batch = []
-        batch.append(await self.queue.get())
+        first_item = await self.queue.get()  # Block until first item arrives
 
         # Cache current max_batch_size and batch_wait_timeout_s for this batch.
         max_batch_size = self.max_batch_size
         batch_wait_timeout_s = self.batch_wait_timeout_s
 
+        # Check if first item alone exceeds max_batch_size (only with batch_size_fn)
+        if self.batch_size_fn is not None:
+            first_item_size = self._compute_batch_size([first_item])
+            if first_item_size > max_batch_size:
+                exc = RuntimeError(
+                    "Size of item is greater than max_batch_size. "
+                    "Please increase the max_batch_size or check the "
+                    "implementation of the batch_size_fn."
+                )
+                # Set exception on the future so the caller receives it
+                first_item.future.set_exception(exc)
+                return [], 0
+
+        batch.append(first_item)
+
         # Wait self.timeout_s seconds for new queue arrivals.
         batch_start_time = time.time()
         while True:
+            # Record queue length metric.
+            self._batch_queue_length_gauge.set(
+                self.queue.qsize(), tags={"function_name": self._function_name}
+            )
+
             remaining_batch_time_s = max(
                 batch_wait_timeout_s - (time.time() - batch_start_time), 0
             )
@@ -206,9 +299,39 @@ class _BatchQueue:
             except asyncio.TimeoutError:
                 pass
 
-            # Add all new arrivals to the batch.
-            while len(batch) < max_batch_size and not self.queue.empty():
-                batch.append(self.queue.get_nowait())
+            # Custom batch size function logic
+            if self.batch_size_fn is not None:
+                # Add all new arrivals to the batch.
+                # Track items we need to put back if they don't fit
+                deferred_item = None
+                while not self.queue.empty():
+                    next_item = self.queue.get_nowait()
+                    # Temporarily add to check size
+                    batch.append(next_item)
+                    new_size = self._compute_batch_size(batch)
+
+                    if new_size > max_batch_size:
+                        # Would exceed limit, remove it and save for later
+                        batch.pop()
+                        deferred_item = next_item
+                        break
+                    # Size is OK, keep it in the batch (already added above)
+
+                # Put deferred item back in queue for next batch
+                if deferred_item is not None:
+                    # NOTE: The deferred item goes to the back of the queue (FIFO),
+                    # so newer requests may be processed before it. Consider using
+                    # asyncio.PriorityQueue if strict ordering is required.
+                    self.queue.put_nowait(deferred_item)
+                    # Compute final batch size before breaking (batch is now valid
+                    # after popping the deferred item).
+                    current_batch_size = self._compute_batch_size(batch)
+                    # break the loop early because the deferred item is too large to fit in the batch
+                    break
+            else:
+                # Default behavior: use original len() check logic
+                while len(batch) < max_batch_size and not self.queue.empty():
+                    batch.append(self.queue.get_nowait())
 
             # Only clear the put event if the queue is empty. If it's not empty
             # we can start constructing a new batch immediately in the next loop.
@@ -219,13 +342,20 @@ class _BatchQueue:
             if self.queue.empty():
                 self.requests_available_event.clear()
 
+            current_batch_size = self._compute_batch_size(batch)
             if (
                 time.time() - batch_start_time >= batch_wait_timeout_s
-                or len(batch) >= max_batch_size
+                or current_batch_size >= max_batch_size
             ):
                 break
 
-        return batch
+        # Record batch wait time metric (time spent waiting for batch to fill).
+        batch_wait_time_ms = (time.time() - batch_start_time) * 1000
+        self._batch_wait_time_histogram.observe(
+            batch_wait_time_ms, tags={"function_name": self._function_name}
+        )
+
+        return batch, current_batch_size
 
     def _validate_results(
         self, results: Iterable[Any], input_batch_length: int
@@ -311,29 +441,57 @@ class _BatchQueue:
         # So we unset the request context so the current context is not inherited by the task, _process_batch.
         serve.context._unset_request_context()
         while not self._loop.is_closed():
-            batch = await self.wait_for_batch()
-            promise = self._process_batch(func, batch)
+            batch, computed_batch_size = await self.wait_for_batch()
+            promise = self._process_batch(func, batch, computed_batch_size)
             task = asyncio.create_task(promise)
             self.tasks.add(task)
             self.curr_iteration_start_times[task] = time.time()
             task.add_done_callback(self._handle_completed_task)
 
-    async def _process_batch(self, func: Callable, batch: List[_SingleRequest]) -> None:
+    async def _process_batch(
+        self, func: Callable, batch: List[_SingleRequest], computed_batch_size: int
+    ) -> None:
         """Processes queued request batch."""
         # NOTE: this semaphore caps the number of concurrent batches specified by `max_concurrent_batches`
         async with self.semaphore:
             # Remove requests that have been cancelled from the batch. If
             # all requests have been cancelled, simply return and wait for
             # the next batch.
+            original_batch_len = len(batch)
             batch = [req for req in batch if not req.future.cancelled()]
             if len(batch) == 0:
                 return
 
+            # Record batch utilization metric.
+            # Use computed_batch_size from wait_for_batch for efficiency.
+            # If requests were cancelled, we need to recompute since the batch changed.
+            if len(batch) != original_batch_len:
+                computed_batch_size = self._compute_batch_size(batch)
+
+            # Calculate and record batch utilization percentage.
+            batch_utilization_percent = (
+                computed_batch_size / self.max_batch_size
+            ) * 100
+            self._batch_utilization_histogram.observe(
+                batch_utilization_percent, tags={"function_name": self._function_name}
+            )
+
+            # Record actual batch size (number of requests in the batch computed by the batch_size_fn).
+            self._batch_size_histogram.observe(
+                computed_batch_size, tags={"function_name": self._function_name}
+            )
+
+            # Increment batches processed counter.
+            self._batches_processed_counter.inc(
+                tags={"function_name": self._function_name}
+            )
+
             futures = [item.future for item in batch]
 
             # Most of the logic in the function should be wrapped in this try-
             # except block, so the futures' exceptions can be set if an exception
             # occurs. Otherwise, the futures' requests may hang indefinitely.
+            batch_execution_start_time = time.time()
             try:
                 self_arg = batch[0].self_arg
                 args, kwargs = _batch_args_kwargs(
@@ -368,6 +526,14 @@ class _BatchQueue:
 
                 for future in futures:
                     _set_exception_if_not_done(future, e)
+            finally:
+                # Record batch execution time.
+                batch_execution_time_ms = (
+                    time.time() - batch_execution_start_time
+                ) * 1000
+                self._batch_execution_time_histogram.observe(
+                    batch_execution_time_ms, tags={"function_name": self._function_name}
+                )
 
     def _handle_completed_task(self, task: asyncio.Task) -> None:
         self.tasks.remove(task)
@@ -409,12 +575,14 @@ class _LazyBatchQueueWrapper:
         batch_wait_timeout_s: float = 0.0,
         max_concurrent_batches: int = 1,
         handle_batch_func: Optional[Callable] = None,
+        batch_size_fn: Optional[Callable[[List], int]] = None,
     ):
         self._queue: Optional[_BatchQueue] = None
         self.max_batch_size = max_batch_size
         self.batch_wait_timeout_s = batch_wait_timeout_s
         self.max_concurrent_batches = max_concurrent_batches
         self.handle_batch_func = handle_batch_func
+        self.batch_size_fn = batch_size_fn
 
     @property
     def queue(self) -> _BatchQueue:
@@ -428,6 +596,7 @@ class _LazyBatchQueueWrapper:
                 self.batch_wait_timeout_s,
                 self.max_concurrent_batches,
                 self.handle_batch_func,
+                self.batch_size_fn,
             )
         return self._queue
 
@@ -516,6 +685,13 @@ def _validate_max_concurrent_batches(max_concurrent_batches: int) -> None:
         )
 
 
+def _validate_batch_size_fn(batch_size_fn: Optional[Callable[[List], int]]) -> None:
+    if batch_size_fn is not None and not callable(batch_size_fn):
+        raise TypeError(
+            f"batch_size_fn must be a callable or None, got {type(batch_size_fn)}"
+        )
+
+
 SelfType = TypeVar("SelfType", contravariant=True)
 T = TypeVar("T")
 R = TypeVar("R")
@@ -564,6 +740,7 @@ def batch(
     max_batch_size: int = 10,
     batch_wait_timeout_s: float = 0.01,
     max_concurrent_batches: int = 1,
+    batch_size_fn: Optional[Callable[[List], int]] = None,
 ) -> "_BatchDecorator":
     ...
 
@@ -601,6 +778,7 @@ def batch(
     max_batch_size: int = 10,
     batch_wait_timeout_s: float = 0.01,
     max_concurrent_batches: int = 1,
+    batch_size_fn: Optional[Callable[[List], int]] = None,
 ) -> Callable:
     """Converts a function to asynchronously handle batches.
 
@@ -652,6 +830,12 @@ def batch(
             executed concurrently. If the number of concurrent batches exceeds
             this limit, the batch handler will wait for a batch to complete
             before sending the next batch to the underlying function.
+        batch_size_fn: optional function to compute the effective batch size.
+            If provided, this function takes a list of items and returns an
+            integer representing the batch size. This is useful for batching
+            based on custom metrics such as total nodes in graphs, total tokens
+            in sequences, or other domain-specific measures. If None, the batch
+            size is computed as len(batch).
     """
     # `_func` will be None in the case when the decorator is parametrized.
     # See the comment at the end of this function for a detailed explanation.
@@ -667,6 +851,7 @@ def batch(
     _validate_max_batch_size(max_batch_size)
     _validate_batch_wait_timeout_s(batch_wait_timeout_s)
     _validate_max_concurrent_batches(max_concurrent_batches)
+    _validate_batch_size_fn(batch_size_fn)
 
     def _batch_decorator(_func):
         lazy_batch_queue_wrapper = _LazyBatchQueueWrapper(
@@ -674,6 +859,7 @@ def batch(
             batch_wait_timeout_s,
             max_concurrent_batches,
             _func,
+            batch_size_fn,
         )
 
         async def batch_handler_generator(
