@@ -473,6 +473,12 @@ class DeploymentSchema(BaseModel, allow_population_by_field_name=True):
             field for field, value in self.dict().items() if value is not DEFAULT.VALUE
         }
 
+    def is_autoscaling_configured(self) -> bool:
+        return self.num_replicas == "auto" or self.autoscaling_config not in [
+            None,
+            DEFAULT.VALUE,
+        ]
+
 
 def _deployment_info_to_schema(name: str, info: DeploymentInfo) -> DeploymentSchema:
     """Converts a DeploymentInfo object to DeploymentSchema."""
@@ -579,6 +585,13 @@ class ServeApplicationSchema(BaseModel):
         default=None,
         description="Logging config for configuring serve application logs.",
     )
+    external_scaler_enabled: bool = Field(
+        default=False,
+        description=(
+            "If True, indicates that an external autoscaler will manage replica scaling for this application. "
+            "When enabled, Serve's built-in autoscaling cannot be used for any deployments in this application."
+        ),
+    )
 
     @property
     def deployment_names(self) -> List[str]:
@@ -640,6 +653,30 @@ class ServeApplicationSchema(BaseModel):
 
         return v
 
+    @root_validator
+    def validate_external_scaler_and_autoscaling(cls, values):
+        external_scaler_enabled = values.get("external_scaler_enabled", False)
+        deployments = values.get("deployments", [])
+
+        if external_scaler_enabled:
+            deployments_with_autoscaling = []
+            for deployment in deployments:
+                if deployment.is_autoscaling_configured():
+                    deployments_with_autoscaling.append(deployment.name)
+
+            if deployments_with_autoscaling:
+                deployment_names = ", ".join(
+                    f'"{name}"' for name in deployments_with_autoscaling
+                )
+                raise ValueError(
+                    f"external_scaler_enabled is set to True, but the following "
+                    f"deployment(s) have autoscaling configured: {deployment_names}. "
+                    "When using an external autoscaler, Serve's built-in autoscaling must "
+                    "be disabled for all deployments in the application."
+                )
+
+        return values
+
     @staticmethod
     def get_empty_schema_dict() -> Dict:
         """Returns an empty app schema dictionary.
@@ -1109,6 +1146,46 @@ class APIType(str, Enum):
         return [cls.IMPERATIVE.value, cls.DECLARATIVE.value]
 
 
+@PublicAPI(stability="alpha")
+class DeploymentNode(BaseModel):
+    """Represents a node in the deployment topology.
+
+    Each node represents a deployment and tracks which other deployments it calls.
+    """
+
+    name: str = Field(description="The name of the deployment.")
+    app_name: str = Field(
+        description="The name of the application this deployment belongs to."
+    )
+    # using name and app_name instead of just deployment name because outbound dependencies can be in different apps
+    outbound_deployments: List[dict] = Field(
+        default_factory=list,
+        description="The deployment IDs that this deployment calls (outbound dependencies).",
+    )
+    is_ingress: bool = Field(
+        default=False, description="Whether this is the ingress deployment."
+    )
+
+
+@PublicAPI(stability="alpha")
+class DeploymentTopology(BaseModel):
+    """Represents the dependency graph of deployments in an application.
+
+    The topology shows which deployments call which other deployments,
+    with the ingress deployment as the entry point.
+    """
+
+    app_name: str = Field(
+        description="The name of the application this topology belongs to."
+    )
+    nodes: Dict[str, DeploymentNode] = Field(
+        description="The adjacency list of deployment nodes."
+    )
+    ingress_deployment: Optional[str] = Field(
+        default=None, description="The name of the ingress deployment (entry point)."
+    )
+
+
 @PublicAPI(stability="stable")
 class ApplicationDetails(BaseModel, extra=Extra.forbid, frozen=True):
     """Detailed info about a Serve application."""
@@ -1167,11 +1244,19 @@ class ApplicationDetails(BaseModel, extra=Extra.forbid, frozen=True):
     deployments: Dict[str, DeploymentDetails] = Field(
         description="Details about the deployments in this application."
     )
+    external_scaler_enabled: bool = Field(
+        description="Whether external scaling is enabled for this application.",
+    )
 
     application_details_route_prefix_format = validator(
         "route_prefix", allow_reuse=True
     )(_route_prefix_format)
 
+    deployment_topology: Optional[DeploymentTopology] = Field(
+        default=None,
+        description="The deployment topology showing how deployments in this application call each other.",
+    )
+
 
 @PublicAPI(stability="stable")
 class ProxyDetails(ServeActorDetails, frozen=True):
