@@ -1,13 +1,14 @@
-import subprocess
-import os
-import sys
-import random
-import threading
 import collections
 import logging
+import os
+import random
 import shutil
+import subprocess
+import sys
+import threading
 import time
 
+from ray._common.network_utils import is_ipv6
 
 _logger = logging.getLogger("ray.util.spark.utils")
 
@@ -100,7 +101,11 @@ def is_port_in_use(host, port):
     import socket
     from contextlib import closing
 
-    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
+    with closing(
+        socket.socket(
+            socket.AF_INET6 if is_ipv6(host) else socket.AF_INET, socket.SOCK_STREAM
+        )
+    ) as sock:
         return sock.connect_ex((host, port)) == 0
 
 
@@ -199,9 +204,10 @@ _RAY_ON_SPARK_NODE_MEMORY_BUFFER_OFFSET = 0.8
 
 
 def calc_mem_ray_head_node(configured_heap_memory_bytes, configured_object_store_bytes):
-    import psutil
     import shutil
 
+    import psutil
+
     if RAY_ON_SPARK_DRIVER_PHYSICAL_MEMORY_BYTES in os.environ:
         available_physical_mem = int(
             os.environ[RAY_ON_SPARK_DRIVER_PHYSICAL_MEMORY_BYTES]
