@@ -1191,6 +1191,7 @@ def get_log(
     submission_id: Optional[str] = None,
     attempt_number: int = 0,
     _interval: Optional[float] = None,
+    filter_ansi_code: bool = False,
 ) -> Generator[str, None, None]:
     """Retrieve log file based on file name or some entities ids (pid, actor id, task id).
 
@@ -1249,6 +1250,8 @@ def get_log(
         submission_id: Job submission ID if getting log from a submission job.
         attempt_number: The attempt number of the task if getting logs generated by a task.
         _interval: The interval in secs to print new logs when `follow=True`.
+        filter_ansi_code: A boolean flag for determining whether to filter ANSI escape codes.
+            Setting to `True` removes ANSI escape codes from the output. The default value is `False`.
 
     Return:
         A Generator of log line, None for SendType and ReturnType.
@@ -1281,6 +1284,9 @@ def get_log(
         if option_val is not None:
             options_dict[field.name] = option_val
 
+    if filter_ansi_code is not None:
+        options_dict["filter_ansi_code"] = filter_ansi_code
+
     with requests.get(
         f"{api_server_url}/api/v0/logs/{media_type}?"
         f"{urllib.parse.urlencode(options_dict)}",
